# -*- coding: utf-8 -*-
"""Definitions of midi events, controller numbers and parameters."""

###################################################
# Midi channel events (the most common events)
# Also called "Channel Voice Messages"

NOTE_OFF = 0x80
# 1000cccc 0nnnnnnn 0vvvvvvv (channel, note, velocity)

NOTE_ON = 0x90
# 1001cccc 0nnnnnnn 0vvvvvvv (channel, note, velocity)

POLY_AFTERTOUCH = POLYPHONIC_PRESSURE = POLY_PRESSURE = 0xA0
# 1010cccc 0nnnnnnn 0vvvvvvv (channel, note, velocity)

# see Channel Mode Messages for Controller Numbers
CONTROLLER_CHANGE = CONTROL_CHANGE = 0xB0
# 1011cccc 0ccccccc 0vvvvvvv (channel, controller, value)

PROGRAM_CHANGE = 0xC0
# 1100cccc 0ppppppp (channel, program)

CHANNEL_AFTERTOUCH = CHANNEL_PRESSURE = MONO_PRESSURE = 0xD0
# 1101cccc 0ppppppp (channel, pressure)

PITCH_BEND = 0xE0
# 1110cccc 0vvvvvvv 0wwwwwww (channel, value-lo, value-hi)


###################################################
# Channel Mode Messages (Continuous Controller)
# All CCs have the same status byte (0xBn).
# The controller number is the first data byte

# High resolution continuous controllers (MSB)

BANK_SELECT = BANK_SELECT_MSB = 0x00
MODULATION = MODULATION_WHEEL = MODULATION_WHEEL_MSB = 0x01
BREATH_CONTROLLER = BREATH_CONTROLLER_MSB = 0x02
FOOT_CONTROLLER = FOOT_CONTROLLER_MSB = 0x04
PORTAMENTO_TIME = PORTAMENTO_TIME_MSB = 0x05
DATA_ENTRY = DATA_ENTRY_MSB = 0x06
VOLUME = CHANNEL_VOLUME = CHANNEL_VOLUME_MSB = 0x07
BALANCE = BALANCE_MSB = 0x08
PAN = PAN_MSB = 0x0A
EXPRESSION = EXPRESSION_CONTROLLER = EXPRESSION_CONTROLLER_MSB = 0x0B
EFFECT_CONTROL_1 = EFFECT_CONTROL_1_MSB = 0x0C
EFFECT_CONTROL_2 = EFFECT_CONTROL_2_MSB = 0x0D
GENERAL_PURPOSE_CONTROLLER_1 = GENERAL_PURPOSE_CONTROLLER_1_MSB = 0x10
GENERAL_PURPOSE_CONTROLLER_2 = GENERAL_PURPOSE_CONTROLLER_2_MSB = 0x11
GENERAL_PURPOSE_CONTROLLER_3 = GENERAL_PURPOSE_CONTROLLER_3_MSB = 0x12
GENERAL_PURPOSE_CONTROLLER_4 = GENERAL_PURPOSE_CONTROLLER_4_MSB = 0x13

# High resolution continuous controllers (LSB)

BANK_SELECT_LSB = 0x20
MODULATION_LSB = MODULATION_WHEEL_LSB = 0x21
BREATH_CONTROLLER_LSB = 0x22
FOOT_CONTROLLER_LSB = 0x24
PORTAMENTO_TIME_LSB = 0x25
DATA_ENTRY_LSB = 0x26
VOLUME_LSB = CHANNEL_VOLUME_LSB = 0x27
BALANCE_LSB = 0x28
PAN_LSB = 0x2A
EXPRESSION_LSB = EXPRESSION_CONTROLLER_LSB = 0x2B
EFFECT_CONTROL_1_LSB = 0x2C
EFFECT_CONTROL_2_LSB = 0x2D
GENERAL_PURPOSE_CONTROLLER_1_LSB = 0x30
GENERAL_PURPOSE_CONTROLLER_2_LSB = 0x31
GENERAL_PURPOSE_CONTROLLER_3_LSB = 0x32
GENERAL_PURPOSE_CONTROLLER_4_LSB = 0x33

# Switches

# off: value <= 63, on: value >= 64
SUSTAIN = SUSTAIN_ONOFF = 0x40
PORTAMENTO = PORTAMENTO_ONOFF = 0x41
SOSTENUTO = SOSTENUTO_ONOFF = 0x42
SOFT_PEDAL = SOFT_PEDAL_ONOFF = 0x43
LEGATO = LEGATO_ONOFF = 0x44
HOLD_2 = HOLD_2_ONOFF = 0x45

# Low resolution continuous controllers

# Sound Variation; FX: Exciter On/Off
SOUND_CONTROLLER_1 = 0x46
# Harmonic Content; FX: Compressor On/Off
SOUND_CONTROLLER_2 = 0x47
# Release Time; FX: Distortion On/Off
SOUND_CONTROLLER_3 = 0x48
# Attack Time; FX: EQ On/Off
SOUND_CONTROLLER_4 = 0x49
# Brightness; FX: Expander On/Off
SOUND_CONTROLLER_5 = 0x4A
# Decay Time; FX: Reverb On/Off
SOUND_CONTROLLER_6 = 0x4B
# Vibrato Rate; FX: Delay On/Off
SOUND_CONTROLLER_7 = 0x4C
# Vibrato Depth; FX: Pitch Transpose On/Off
SOUND_CONTROLLER_8 = 0x4D
# Vibrato Delay; FX: Flange/Chorus On/Off
SOUND_CONTROLLER_9 = 0x4E
# Undefined; FX: Special Effects On/Off
SOUND_CONTROLLER_10 = 0x4F
GENERAL_PURPOSE_CONTROLLER_5 = 0x50
GENERAL_PURPOSE_CONTROLLER_6 = 0x51
GENERAL_PURPOSE_CONTROLLER_7 = 0x52
GENERAL_PURPOSE_CONTROLLER_8 = 0x53
# PTC, 0vvvvvvv is the source Note number
PORTAMENTO_CONTROL = PTC = 0x54
HIGH_RESOLUTION_VELOCITY_PREFIX = 0x58
# Reverb Send Level, formerly Ext. Effects Depth
EFFECTS_1 = EFFECTS_1_DEPTH = 0x5B
# formerly Tremelo Depth
EFFECTS_2 = EFFECTS_2_DEPTH = 0x5C
# Chorus Send Level, formerly Chorus Depth
EFFECTS_3 = EFFECTS_3_DEPTH = 0x5D
# formerly Celeste(Detune) Depth
EFFECTS_4 = EFFECTS_4_DEPTH = 0x5E
# formerly Phaser Depth
EFFECTS_5 = EFFECTS_5_DEPTH = 0x5F
# controller value byte should be 0
DATA_INCREMENT = 0x60
# controller value byte should be 0
DATA_DECREMENT = 0x61
NRPN_LSB = NON_REGISTERED_PARAMETER_NUMBER_LSB = 0x62
NRPN_MSB = NON_REGISTERED_PARAMETER_NUMBER_MSB = 0x63
RPN_LSB = REGISTERED_PARAMETER_NUMBER_LSB = 0x64
RPN_MSB = REGISTERED_PARAMETER_NUMBER_MSB = 0x65

# Channel Mode messages

# controller value byte should be 0
ALL_SOUND_OFF = 0x78
# controller value byte should be 0
RESET_ALL_CONTROLLERS = 0x79
# 0 = off, 127 = on
LOCAL_CONTROL = LOCAL_CONTROL_ONOFF = 0x7A
# controller value byte should be 0
ALL_NOTES_OFF = 0x7B
# controller value byte should be 0, also causes ANO
OMNI_MODE_OFF = 0x7C
# controller value byte should be 0, also causes ANO
OMNI_MODE_ON = 0x7D
# Mono Mode on / Poly Off; also causes ANO
# 1011nnnn 01111110 0000vvvv
# vvvv > 0 : Number of channels to use (Omni Off).
# vvvv = 0 : Use all available channels (Omni On)
MONO_MODE_ON = 0x7E
# Poly Mode On / Mono Off
# controller value byte should be 0, also causes ANO
POLY_MODE_ON = 0x7F


###################################################
# System Common Messages, for all channels

# 11110000 0iiiiiii 0ddddddd ... 11110111
SYSTEM_EXCLUSIVE = 0xF0

# MIDI Time Code Quarter Frame
# 11110001
MIDI_TIME_CODE = MTC = 0xF1

# 11110010 0vvvvvvv 0wwwwwww (lo-position, hi-position)
SONG_POSITION_POINTER = 0xF2

# 11110011 0sssssss (songnumber)
SONG_SELECT = 0xF3

# 11110100 (0xF4) is undefined
# 11110101 (0xF5) is undefined

# 11110110
TUNING_REQUEST = TUNE_REQUEST = 0xF6

# 11110111 # End of system exclusive
END_OF_EXCLUSIVE = 0xF7


###################################################
# Midifile meta-events

SEQUENCE_NUMBER = 0x00  # 00 02 ss ss (seq-number)
TEXT = 0x01             # 01 len text...
COPYRIGHT = 0x02        # 02 len text...
SEQUENCE_NAME = 0x03    # 03 len text...
INSTRUMENT_NAME = 0x04  # 04 len text...
LYRIC = 0x05            # 05 len text...
MARKER = 0x06           # 06 len text...
CUEPOINT = 0x07         # 07 len text...
PROGRAM_NAME = 0x08     # 08 len text...
DEVICE_NAME = 0x09      # 09 len text...

MIDI_CH_PREFIX = 0x20   # MIDI channel prefix assignment (deprecated)

MIDI_PORT = 0x21        # 21 01 port, deprecated but still used
END_OF_TRACK = 0x2F     # 2f 00
TEMPO = 0x51            # 51 03 tt tt tt (tempo in µs/quarternote)
SMTP_OFFSET = 0x54      # 54 05 hh mm ss ff xx
TIME_SIGNATURE = 0x58   # 58 04 nn dd cc bb
KEY_SIGNATURE = 0x59    # 59 02 sf mi (sf = number of sharps(+) or flats(-)
                        # mi = major(0) or minor (1))
SPECIFIC = 0x7F         # Sequencer specific event


###################################################
# System Realtime messages
# These should not occur in midi files

TIMING_CLOCK = 0xF8
# 0xF9 is undefined
SONG_START = 0xFA
SONG_CONTINUE = 0xFB
SONG_STOP = 0xFC
# 0xFD is undefined
ACTIVE_SENSING = 0xFE
SYSTEM_RESET = 0xFF


###################################################
# META EVENT, it is used only in midi files.
# In transmitted data it means system reset!!!

# 11111111
META_EVENT = 0xFF
ESCAPE_SEQUENCE = 0xF7


###################################################
# Misc constants

FILE_HEADER = 'MThd'
TRACK_HEADER = 'MTrk'

# Timecode resolution: frames per second
FPS_24 = 0xE8
FPS_25 = 0xE7
FPS_29 = 0xE3
FPS_30 = 0xE2


###################################################
# Helper functions

def is_status(byte):
    """Return True if the given byte is a MIDI status byte, False otherwise."""
    return (byte & 0x80) == 0x80  # 1000 0000



#!/usr/bin/env python
#
# midiin_callback.py
#
"""Show how to receive MIDI input by setting a callback function."""

from __future__ import print_function

import logging
import sys
import time

from rtmidi.midiutil import open_midiinput

log = logging.getLogger('midiin_callback')
logging.basicConfig(level=logging.DEBUG)


class MidiInputHandler(object):
    def __init__(self, port):
        self.port = port
        self._wallclock = time.time()

    def __call__(self, event, data=None):
        message, deltatime = event
        self._wallclock += deltatime
        print("[%s] @%0.6f %r" % (self.port, self._wallclock, message))


# Prompts user for MIDI input port, unless a valid port number or name
# is given as the first argument on the command line.
# API backend defaults to ALSA on Linux.
port = sys.argv[1] if len(sys.argv) > 1 else None

try:
    midiin, port_name = open_midiinput(port)
except (EOFError, KeyboardInterrupt):
    sys.exit()

print("Attaching MIDI input callback handler.")
midiin.set_callback(MidiInputHandler(port_name))

print("Entering main loop. Press Control-C to exit.")
try:
    # Just wait for keyboard interrupt,
    # everything else is handled via the input callback.
    while True:
        time.sleep(1)
except KeyboardInterrupt:
    print('')
finally:
    print("Exit.")
    midiin.close_port()
    del midiin



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Unit tests for the rtmidi module."""

import time
import unittest

import rtmidi


if bytes is str:
    string_types = (str, unicode)  # noqa:F821
else:
    string_types = (str,)


class StaticFunctionsTests(unittest.TestCase):
    def test_get_api_display_name(self):
        # Why not 'Unspecified'?
        self.assertEqual(rtmidi.get_api_display_name(rtmidi.API_LINUX_ALSA), 'ALSA')
        self.assertEqual(rtmidi.get_api_display_name(rtmidi.API_MACOSX_CORE), 'CoreMidi')
        self.assertEqual(rtmidi.get_api_display_name(rtmidi.API_RTMIDI_DUMMY), 'Dummy')
        self.assertEqual(rtmidi.get_api_display_name(rtmidi.API_UNIX_JACK), 'Jack')
        self.assertEqual(rtmidi.get_api_display_name(rtmidi.API_UNSPECIFIED), 'Unknown')
        self.assertEqual(rtmidi.get_api_display_name(rtmidi.API_WINDOWS_MM), 'Windows MultiMedia')

    def test_get_api_name(self):
        self.assertEqual(rtmidi.get_api_name(rtmidi.API_LINUX_ALSA), 'alsa')
        self.assertEqual(rtmidi.get_api_name(rtmidi.API_MACOSX_CORE), 'core')
        self.assertEqual(rtmidi.get_api_name(rtmidi.API_RTMIDI_DUMMY), 'dummy')
        self.assertEqual(rtmidi.get_api_name(rtmidi.API_UNIX_JACK), 'jack')
        self.assertEqual(rtmidi.get_api_name(rtmidi.API_UNSPECIFIED), 'unspecified')
        self.assertEqual(rtmidi.get_api_name(rtmidi.API_WINDOWS_MM), 'winmm')

    def test_get_compiled_api(self):
        apilist = rtmidi.get_compiled_api()
        self.assertTrue(isinstance(apilist, list))
        self.assertTrue(len(apilist) >= 1)
        for api in apilist:
            self.assertTrue(api <= rtmidi.API_RTMIDI_DUMMY)

    def test_get_compiled_api_by_name(self):
        for api, name in (
                (rtmidi.API_LINUX_ALSA, 'alsa'),
                (rtmidi.API_MACOSX_CORE, 'core'),
                (rtmidi.API_RTMIDI_DUMMY, 'dummy'),
                (rtmidi.API_UNIX_JACK, 'jack'),
                (rtmidi.API_WINDOWS_MM, 'winmm')):

            res = rtmidi.get_compiled_api_by_name(name)

            if api in rtmidi.get_compiled_api():
                self.assertEqual(res, api)
            else:
                self.assertEqual(res, rtmidi.API_UNSPECIFIED)

    def test_get_rtmidi_version(self):
        version = rtmidi.get_rtmidi_version()
        self.assertTrue(isinstance(version, string_types))
        self.assertEqual(version, '4.0.0')


class BaseTests:
    NOTE_ON = [0x90, 48, 100]
    NOTE_OFF = [0x80, 48, 16]
    IN_CLIENT_NAME = "RtMidiTestCase In"
    OUT_CLIENT_NAME = "RtMidiTestCase Out"
    IN_PORT_NAME = 'testin'
    OUT_PORT_NAME = 'testout'
    DELAY = 0.1
    API = rtmidi.API_UNSPECIFIED

    def setUp(self):
        self.midi_in = rtmidi.MidiIn(rtapi=self.API, name=self.IN_CLIENT_NAME)
        self.midi_out = rtmidi.MidiOut(rtapi=self.API, name=self.OUT_CLIENT_NAME)

    def tearDown(self):
        self.midi_in.close_port()
        del self.midi_in
        self.midi_out.close_port()
        del self.midi_out

    def test_is_port_open(self):
        assert not self.midi_in.is_port_open()
        self.midi_in.open_port(0)
        assert self.midi_in.is_port_open()
        self.midi_in.close_port()
        assert not self.midi_in.is_port_open()

        assert not self.midi_out.is_port_open()
        self.midi_out.open_port(0)
        assert self.midi_out.is_port_open()
        self.midi_out.close_port()
        assert not self.midi_out.is_port_open()

    def test_get_current_api(self):
        assert self.midi_in.get_current_api() == self.API
        assert self.midi_out.get_current_api() == self.API


class VirtualPortsSupportedTests:
    def test_is_port_open_virtual(self):
        # virtual ports can't be closed
        assert not self.midi_in.is_port_open()
        self.midi_in.open_virtual_port()
        assert self.midi_in.is_port_open()
        self.midi_in.close_port()
        assert self.midi_in.is_port_open()

        assert not self.midi_out.is_port_open()
        self.midi_out.open_virtual_port()
        assert self.midi_out.is_port_open()
        self.midi_out.close_port()
        assert self.midi_out.is_port_open()

    def test_send_and_get_message(self):
        self.set_up_loopback()
        self.midi_out.send_message(self.NOTE_ON)
        self.midi_out.send_message(self.NOTE_OFF)
        time.sleep(self.DELAY)
        message_1, _ = self.midi_in.get_message()
        message_2, _ = self.midi_in.get_message()
        self.assertEqual(message_1, self.NOTE_ON)
        self.assertEqual(message_2, self.NOTE_OFF)

    def test_callback(self):
        messages = []

        def callback(event, data):
            messages.append((event[0], data))

        self.set_up_loopback()
        self.midi_in.set_callback(callback, data=42)
        self.midi_out.send_message(self.NOTE_ON)
        self.midi_out.send_message(self.NOTE_OFF)
        time.sleep(self.DELAY)
        self.assertEqual(messages[0], (self.NOTE_ON, 42))
        self.assertEqual(messages[1], (self.NOTE_OFF, 42))

        self.midi_in.cancel_callback()
        messages = []
        self.midi_out.send_message(self.NOTE_ON)
        self.midi_out.send_message(self.NOTE_OFF)
        time.sleep(self.DELAY)
        self.assertEqual(messages, [])

    def set_up_loopback(self):
        # TODO: find better solution than this hack-ish strategy to find out
        # the port number of the virtual output port, which we have to use,
        # because for ALSA virtual ports, their name includes the client id.
        # See: https://github.com/thestk/rtmidi/issues/88
        ports_before = self.midi_in.get_ports()
        self.midi_out.open_virtual_port(name=self.OUT_PORT_NAME)
        ports_after = self.midi_in.get_ports()
        self.midi_out_port_name = list(set(ports_after).difference(ports_before))[0]

        for portnum, port in enumerate(ports_after):
            if port == self.midi_out_port_name:
                self.midi_in.open_port(portnum)
                break
        else:
            raise IOError("Could not find MIDI output port.")


class VirtualPortsUnsupportedTests:
    def test_is_port_open_virtual(self):
        # virtual ports can't be closed
        assert not self.midi_in.is_port_open()
        self.assertRaises(NotImplementedError, self.midi_in.open_virtual_port)


class SetPortNameSupportedTests:
    def test_set_port_name_supported(self):
        self.midi_out.open_virtual_port(name=self.OUT_PORT_NAME)
        found = False
        for port in self.midi_in.get_ports():
            client, port = port.split(':', 1)
            if client.startswith(self.OUT_CLIENT_NAME) and port.startswith(self.OUT_PORT_NAME):
                found = True
                break

        assert found

        self.midi_out.set_port_name("new_port")

        found = False
        for port in self.midi_in.get_ports():
            client, port = port.split(':', 1)
            if client.startswith(self.OUT_CLIENT_NAME) and port.startswith("new_port"):
                found = True
                break

        assert found


class SetPortNameUnsupportedTests:
    def test_set_port_name_unsupported(self):
        self.assertRaises(NotImplementedError, self.midi_out.set_port_name, "new_port")


class SetClientNameSupportedTests:
    def test_set_client_name_supported(self):
        self.midi_out.open_virtual_port(name=self.OUT_PORT_NAME)
        found = False
        for port in self.midi_in.get_ports():
            client, port = port.split(':', 1)
            if client.startswith(self.OUT_CLIENT_NAME) and port.startswith(self.OUT_PORT_NAME):
                found = True
                break

        assert found

        self.midi_out.set_client_name("new_client")

        found = False
        for port in self.midi_in.get_ports():
            client, port = port.split(':', 1)
            if client.startswith("new_client") and port.startswith(self.OUT_PORT_NAME):
                found = True
                break

        assert found


class SetClientNameUnsupportedTests:
    def test_set_client_name_unsupported(self):
        self.assertRaises(NotImplementedError, self.midi_out.set_client_name, "new_client")


if rtmidi.API_LINUX_ALSA in rtmidi.get_compiled_api():
    class ALSATestCase(BaseTests, SetPortNameSupportedTests, SetClientNameSupportedTests,
                       VirtualPortsSupportedTests, unittest.TestCase):
        API = rtmidi.API_LINUX_ALSA


if rtmidi.API_UNIX_JACK in rtmidi.get_compiled_api():
    class JACKTestCase(BaseTests, SetPortNameSupportedTests, SetClientNameUnsupportedTests,
                       VirtualPortsSupportedTests, unittest.TestCase):
        API = rtmidi.API_UNIX_JACK


if rtmidi.API_MACOSX_CORE in rtmidi.get_compiled_api():
    class CoreMIDITestCase(BaseTests, SetPortNameUnsupportedTests, SetClientNameUnsupportedTests,
                           VirtualPortsSupportedTests, unittest.TestCase):
        API = rtmidi.API_MACOSX_CORE


if rtmidi.API_WINDOWS_MM in rtmidi.get_compiled_api():
    class WindowsMMTestCase(BaseTests, SetPortNameUnsupportedTests, SetClientNameUnsupportedTests,
                            VirtualPortsUnsupportedTests, unittest.TestCase):
        API = rtmidi.API_WINDOWS_MM


if __name__ == '__main__':
    unittest.main()



# -*- encoding: utf-8 -*-
#cython: embedsignature=True
#cython: language_level=3
#
# rtmidi.pyx
#
"""A Python binding for the RtMidi C++ library implemented using Cython.

Overview
========

**RtMidi** is a set of C++ classes which provides a concise and simple,
cross-platform API (Application Programming Interface) for realtime MIDI
input / output across Linux (ALSA & JACK), macOS / OS X (CoreMIDI & JACK),
and Windows (MultiMedia System) operating systems.

**python-rtmidi** is a Python binding for RtMidi implemented using Cython_ and
provides a thin wrapper around the RtMidi C++ interface. The API is basically
the same as the C++ one but with the naming scheme of classes, methods and
parameters adapted to the Python PEP-8 conventions and requirements of the
Python package naming structure. **python-rtmidi** supports Python 3 (3.5, 3.6,
3.7, and 3.8).


Usage example
=============

Here's a short example of how to use **python-rtmidi** to open the first
available MIDI output port and send a middle C note on MIDI channel 1::

    import time
    import rtmidi

    midiout = rtmidi.MidiOut()
    available_ports = midiout.get_ports()

    if available_ports:
        midiout.open_port(0)
    else:
        midiout.open_virtual_port("My virtual output")

    with midiout:
        note_on = [0x90, 60, 112] # channel 1, middle C, velocity 112
        note_off = [0x80, 60, 0]
        midiout.send_message(note_on)
        time.sleep(0.5)
        midiout.send_message(note_off)
        time.sleep(0.1)

    del midiout


Constants
=========


Low-level APIs
--------------

These constants are returned by the ``get_compiled_api`` function and the
``MidiIn.get_current_api`` resp. ``MidiOut.get_current_api`` methods and are
used to specify the low-level MIDI backend API to use when creating a
``MidiIn`` or ``MidiOut`` instance.

``API_UNSPECIFIED``
    Use first compiled-in API, which has any input resp. output ports
``API_MACOSX_CORE``
    macOS (OS X) CoreMIDI
``API_LINUX_ALSA``
    Linux ALSA
``API_UNIX_JACK``
    Jack Client
``API_WINDOWS_MM``
    Windows MultiMedia
``API_RTMIDI_DUMMY``
    RtMidi Dummy API (used when no suitable API was found)


Error types
-----------

These constants are passed as the first argument to an error handler
function registered with ``set_error_callback`` method of a ``MidiIn``
or ``MidiOut`` instance. For the meaning of each value, please see
the `RtMidi API reference`_.

* ``ERRORTYPE_DEBUG_WARNING``
* ``ERRORTYPE_DRIVER_ERROR``
* ``ERRORTYPE_INVALID_DEVICE``
* ``ERRORTYPE_INVALID_PARAMETER``
* ``ERRORTYPE_INVALID_USE``
* ``ERRORTYPE_MEMORY_ERROR``
* ``ERRORTYPE_NO_DEVICES_FOUND``
* ``ERRORTYPE_SYSTEM_ERROR``
* ``ERRORTYPE_THREAD_ERROR``
* ``ERRORTYPE_UNSPECIFIED``
* ``ERRORTYPE_WARNING``


.. _cython: http://cython.org/
.. _rtmidi api reference:
    http://www.music.mcgill.ca/~gary/rtmidi/classRtMidiError.html

"""

import sys
import warnings

from libcpp cimport bool
from libcpp.string cimport string
from libcpp.vector cimport vector


__all__ = (
    'API_UNSPECIFIED', 'API_MACOSX_CORE', 'API_LINUX_ALSA', 'API_UNIX_JACK',
    'API_WINDOWS_MM', 'API_RTMIDI_DUMMY', 'ERRORTYPE_DEBUG_WARNING',
    'ERRORTYPE_DRIVER_ERROR', 'ERRORTYPE_INVALID_DEVICE',
    'ERRORTYPE_INVALID_PARAMETER', 'ERRORTYPE_INVALID_USE',
    'ERRORTYPE_MEMORY_ERROR', 'ERRORTYPE_NO_DEVICES_FOUND',
    'ERRORTYPE_SYSTEM_ERROR', 'ERRORTYPE_THREAD_ERROR',
    'ERRORTYPE_UNSPECIFIED', 'ERRORTYPE_WARNING', 'InvalidPortError',
    'InvalidUseError', 'MemoryAllocationError', 'MidiIn', 'MidiOut',
    'NoDevicesError', 'RtMidiError', 'SystemError',
    'UnsupportedOperationError', 'get_api_display_name', 'get_api_name',
    'get_compiled_api', 'get_compiled_api_by_name', 'get_rtmidi_version'
)

if bytes is str:
    string_types = (str, unicode)
else:
    string_types = (str,)


# Init Python threads and GIL, because RtMidi calls Python from native threads.
# See http://permalink.gmane.org/gmane.comp.python.cython.user/5837
cdef extern from "Python.h":
    void PyEval_InitThreads()

PyEval_InitThreads()


# Declarations for RtMidi C++ classes and their methods we use

cdef extern from "RtMidi.h":
    # Enums nested in classes are apparently not supported by Cython yet
    # therefore we need to use the following work-around.
    # https://groups.google.com/d/msg/cython-users/monwJxJCb-g/k_h1rcU-3TgJ
    cdef enum Api "RtMidi::Api":
        UNSPECIFIED  "RtMidi::UNSPECIFIED"
        MACOSX_CORE  "RtMidi::MACOSX_CORE"
        LINUX_ALSA   "RtMidi::LINUX_ALSA"
        UNIX_JACK    "RtMidi::UNIX_JACK"
        WINDOWS_MM   "RtMidi::WINDOWS_MM"
        RTMIDI_DUMMY "RtMidi::RTMIDI_DUMMY"

    cdef enum ErrorType "RtMidiError::Type":
        ERR_WARNING           "RtMidiError::WARNING"
        ERR_DEBUG_WARNING     "RtMidiError::DEBUG_WARNING"
        ERR_UNSPECIFIED       "RtMidiError::UNSPECIFIED"
        ERR_NO_DEVICES_FOUND  "RtMidiError::NO_DEVICES_FOUND"
        ERR_INVALID_DEVICE    "RtMidiError::INVALID_DEVICE"
        ERR_MEMORY_ERROR      "RtMidiError::MEMORY_ERROR"
        ERR_INVALID_PARAMETER "RtMidiError::INVALID_PARAMETER"
        ERR_INVALID_USE       "RtMidiError::INVALID_USE"
        ERR_DRIVER_ERROR      "RtMidiError::DRIVER_ERROR"
        ERR_SYSTEM_ERROR      "RtMidiError::SYSTEM_ERROR"
        ERR_THREAD_ERROR      "RtMidiError::THREAD_ERROR"

    # Another work-around for calling C++ static methods:
    cdef string RtMidi_getApiDisplayName "RtMidi::getApiDisplayName"(Api rtapi)
    cdef string RtMidi_getApiName "RtMidi::getApiName"(Api rtapi)
    cdef void RtMidi_getCompiledApi "RtMidi::getCompiledApi"(vector[Api] &apis)
    cdef Api RtMidi_getCompiledApiByName "RtMidi::getCompiledApiByName"(string)
    cdef string RtMidi_getVersion "RtMidi::getVersion"()

    ctypedef void (*RtMidiCallback)(double timeStamp,
                                    vector[unsigned char] *message,
                                    void *userData)

    ctypedef void (*RtMidiErrorCallback)(ErrorType errorType,
                                         const string errorText,
                                         void *userData) except *

    cdef cppclass RtMidi:
        void closePort() except *
        unsigned int getPortCount() except *
        string getPortName(unsigned int portNumber) except *
        void openPort(unsigned int portNumber, string &portName) except *
        void openVirtualPort(string portName) except *
        void setClientName(string &clientName) except *
        void setErrorCallback(RtMidiErrorCallback callback, void *userData) except *
        void setPortName(string &portName) except *

    cdef cppclass RtMidiIn(RtMidi):
        Api RtMidiIn(Api rtapi, string clientName,
                     unsigned int queueSizeLimit) except +
        void cancelCallback() except *
        Api getCurrentApi()
        double getMessage(vector[unsigned char] *message) except *
        void ignoreTypes(bool midiSysex, bool midiTime, bool midiSense) except *
        void setCallback(RtMidiCallback callback, void *data) except *

    cdef cppclass RtMidiOut(RtMidi):
        Api RtMidiOut(Api rtapi, string clientName) except +
        Api getCurrentApi()
        void sendMessage(vector[unsigned char] *message) except *


# internal functions

cdef void _cb_func(double delta_time, vector[unsigned char] *msg_v,
                   void *cb_info) with gil:
    """Wrapper for a Python callback function for MIDI input."""
    func, data = (<object> cb_info)
    message = [msg_v.at(i) for i in range(msg_v.size())]
    func((message, delta_time), data)


cdef void _cb_error_func(ErrorType errorType, const string &errorText,
                         void *cb_info) except * with gil:
    """Wrapper for a Python callback function for errors."""
    func, data, decoder = (<object> cb_info)
    func(errorType, decoder(errorText), data)


def _to_bytes(name):
    """Convert a unicode (Python 2) or str (Python 3) object into bytes."""
    # 'bytes' == 'str' in Python 2 but a separate type in Python 3
    if isinstance(name, string_types):
        try:
            name = bytes(name, 'utf-8')  # Python 3
        except TypeError:
            name = name.encode('utf-8')  # Python 2

    if not isinstance(name, bytes):
        raise TypeError("name must be bytes or (unicode) string.")

    return name


# Public API

# export Api enum values to Python

API_UNSPECIFIED = UNSPECIFIED
API_MACOSX_CORE = MACOSX_CORE
API_LINUX_ALSA = LINUX_ALSA
API_UNIX_JACK = UNIX_JACK
API_WINDOWS_MM = WINDOWS_MM
API_RTMIDI_DUMMY = RTMIDI_DUMMY

# export error values to Python

ERRORTYPE_WARNING = ERR_WARNING
ERRORTYPE_DEBUG_WARNING = ERR_DEBUG_WARNING
ERRORTYPE_UNSPECIFIED = ERR_UNSPECIFIED
ERRORTYPE_NO_DEVICES_FOUND = ERR_NO_DEVICES_FOUND
ERRORTYPE_INVALID_DEVICE = ERR_INVALID_DEVICE
ERRORTYPE_MEMORY_ERROR = ERR_MEMORY_ERROR
ERRORTYPE_INVALID_PARAMETER = ERR_INVALID_PARAMETER
ERRORTYPE_INVALID_USE = ERR_INVALID_USE
ERRORTYPE_DRIVER_ERROR = ERR_DRIVER_ERROR
ERRORTYPE_SYSTEM_ERROR = ERR_SYSTEM_ERROR
ERRORTYPE_THREAD_ERROR = ERR_THREAD_ERROR


# custom exceptions

class RtMidiError(Exception):
    """Base general RtMidi exception.

    All other exceptions in this module derive form this exception.

    Instances have a ``type`` attribute that maps to one of the
    ``ERRORTYPE_*`` constants.

    """
    type = ERR_UNSPECIFIED

    def __init__(self, msg, type=None):
        super().__init__(msg)
        self.type = self.type if type is None else type


class InvalidPortError(RtMidiError, ValueError):
    """Raised when an invalid port number is used.

    Also derives from ``ValueError``.

    """
    type = ERR_INVALID_PARAMETER


class InvalidUseError(RtMidiError, RuntimeError):
    """Raised when an method call is not allowed in the current state.

    Also derives from ``RuntimeError``.

    """
    type = ERR_INVALID_USE


class MemoryAllocationError(RtMidiError, MemoryError):
    """Raised if a memory allocation failed on the C++ level.

    Also derives from ``MemoryError``.

    """
    type = ERR_MEMORY_ERROR


class SystemError(RtMidiError, OSError):
    """Raised if an error happened at the MIDI driver or OS level.

    Also derives from ``OSError``.

    """
    pass


class NoDevicesError(SystemError):
    """Raised if no MIDI devices are found.

    Derives from ``rtmidi.SystemError``.

    """
    type = ERR_NO_DEVICES_FOUND


class UnsupportedOperationError(RtMidiError, RuntimeError):
    """Raised if a method is not supported by the low-level API.

    Also derives from ``RuntimeError``.

    """
    pass


# wrappers for RtMidi's static methods and classes

def get_api_display_name(api):
    """Return the display name of a specified MIDI API.

    This returns a long name used for display purposes.

    The ``api`` should be given as the one of ``API_*`` constants in the
    module namespace, e.g.::

        display_name = rtmidi.get_api_display_name(rtmidi.API_UNIX_JACK)

    If the API is unknown, this function will return the empty string.

    """
    return RtMidi_getApiDisplayName(api).decode('utf-8')


def get_api_name(api):
    """Return the name of a specified MIDI API.

    This returns a short lower-case name used for identification purposes.

    The ``api`` should be given as the one of ``API_*`` constants in the
    module namespace, e.g.::

        name = rtmidi.get_api_name(rtmidi.API_UNIX_JACK)

    If the API is unknown, this function will return the empty string.

    """
    return RtMidi_getApiName(api).decode('utf-8')


def get_compiled_api():
    """Return a list of low-level MIDI backend APIs this module supports.

    Check for support for a particular API by using the ``API_*`` constants in
    the module namespace, i.e.::

        if rtmidi.API_UNIX_JACK in rtmidi.get_compiled_api():
            ...

    """
    cdef vector[Api] api_v

    RtMidi_getCompiledApi(api_v)
    return [api_v[i] for i in range(api_v.size())]


def get_compiled_api_by_name(name):
    """Return the compiled MIDI API having the given name.

    A case insensitive comparison will check the specified name against the
    list of compiled APIs, and return the one which matches. On failure, the
    function returns ``API_UNSPECIFIED``.

    """
    return RtMidi_getCompiledApiByName(_to_bytes(name))


def get_rtmidi_version():
    """Return the version string of the wrapped RtMidi library."""
    return RtMidi_getVersion().decode('utf-8')


def _default_error_handler(etype, msg, data=None):
    if etype == ERR_MEMORY_ERROR:
        raise MemoryAllocationError(msg)
    elif etype == ERR_INVALID_PARAMETER:
        raise InvalidPortError(msg)
    elif etype in (ERR_DRIVER_ERROR, ERR_SYSTEM_ERROR, ERR_THREAD_ERROR):
        raise SystemError(msg, type=etype)
    elif etype in (ERR_WARNING, ERR_DEBUG_WARNING):
        if 'portNumber' in msg and msg.endswith('is invalid.'):
            raise InvalidPortError(msg)
        elif msg.endswith('no ports available!'):
            raise InvalidPortError(msg)
        elif msg.endswith('error looking for port name!'):
            raise InvalidPortError(msg)
        elif msg.endswith('event parsing error!'):
            raise ValueError(msg)
        elif msg.endswith('error sending MIDI message to port.'):
            raise SystemError(msg, type=ERR_DRIVER_ERROR)
        elif msg.endswith('error sending MIDI to virtual destinations.'):
            raise SystemError(msg, type=ERR_DRIVER_ERROR)
        elif msg.endswith('JACK server not running?'):
            raise SystemError(msg, type=ERR_DRIVER_ERROR)
        else:
            warnings.warn(msg)
            return

    raise RtMidiError(msg, type=etype)


cdef class MidiBase:
    cdef object _port
    cdef object _error_callback
    cdef object _deleted

    cdef RtMidi* baseptr(self):
        return NULL

    # context management
    def __enter__(self):
        """Support context manager protocol.

        This means you can use ``MidiIn`` / ``MidiOut`` instances like this:

        :

            midiout = MidiIn()
            midiout.open_port(0)

            with midiout:
                midiout.send_message([...])

        and ``midiout.close_port()`` will be called automatically when exiting
        the ``with`` block.

        Since ``open_port()`` also returns the instance, you can even do:

        :

            midiout = MidiIn()

            with midiout.open_port(0):
                midiout.send_message([...])

        """
        return self

    def __exit__(self, *exc_info):
        """Support context manager protocol.

        This method is called when using a ``MidiIn`` / ``MidiOut`` instance as
        a context manager and closes open ports when exiting the ``with`` block.

        """
        self.close_port()

    def _check_port(self):
        inout = "input" if isinstance(self, MidiIn) else "output"
        if self._port == -1:
            raise InvalidUseError("%r already opened virtual %s port." %
                                  (self, inout))
        elif self._port is not None:
            raise InvalidUseError("%r already opened %s port %i." %
                                  (self, inout, self._port))
        return inout

    def _decode_string(self, s, encoding='auto'):
        """Decode given byte string with given encoding."""
        if encoding == 'auto':
            if sys.platform.startswith('win'):
                encoding = 'latin1'
            elif (self.get_current_api() == API_MACOSX_CORE and
                  sys.platform == 'darwin'):
                encoding = 'macroman'
            else:
                encoding = 'utf-8'

        return s.decode(encoding, "ignore")

    def get_port_count(self):
        """Return the number of available MIDI input or output ports."""
        return self.baseptr().getPortCount()

    def get_port_name(self, unsigned int port, encoding='auto'):
        """Return the name of the MIDI input or output port with given number.

        Ports are numbered from zero, separately for input and output ports.
        The number of available ports is returned by the ``get_port_count``
        method.

        The port name is decoded to a (unicode) string with the encoding given
        by ``encoding``. If ``encoding`` is ``"auto"`` (the default), then an
        appropriate encoding is chosen based on the system and the used backend
        API. If ``encoding`` is ``None``, the name is returned un-decoded, i.e.
        as type ``str`` in Python 2 or ``bytes`` in Python 3.

        """
        cdef string name = self.baseptr().getPortName(port)

        if len(name):
            return self._decode_string(name, encoding) if encoding else name

    def get_ports(self, encoding='auto'):
        """Return a list of names of available MIDI input or output ports.

        The list index of each port name corresponds to its port number.

        The port names are decoded to (unicode) strings with the encoding given
        by ``encoding``. If ``encoding`` is ``"auto"`` (the default), then an
        appropriate encoding is chosen based on the system and the used backend
        API. If ``encoding`` is ``None``, the names are returned un-decoded,
        i.e. as type ``str`` in Python 2 or ``bytes`` in Python 3.

        """
        return [self.get_port_name(p, encoding=encoding)
                for p in range(self.get_port_count())]

    def is_port_open(self):
        """Return ``True`` if a port has been opened and ``False`` if not.

        .. note::
            The ``isPortOpen`` method of the RtMidi C++ library does not
            return ``True`` when a virtual port has been openend. The
            python-rtmidi implementation, on the other hand, does.

        """
        return self._port is not None

    def open_port(self, unsigned int port=0, name=None):
        """Open the MIDI input or output port with the given port number.

        Only one port can be opened per ``MidiIn`` or ``MidiOut`` instance. An
        ``InvalidUseError`` exception is raised if an attempt is made to open a
        port on a ``MidiIn`` or ``MidiOut`` instance, which already opened a
        (virtual) port.

        You can optionally pass a name for the RtMidi port with the ``name``
        keyword or the second positional argument. Names with non-ASCII
        characters in them have to be passed as unicode or UTF-8 encoded
        strings in Python 2. The default name is "RtMidi input" resp. "RtMidi
        output".

        .. note::
            Closing a port and opening it again with a different name does not
            change the port name. To change the port name, use the
            ``set_port_name`` method where supported, or delete its instance,
            create a new one and open the port again giving a different name.

        Exceptions:

        ``InvalidPortError``
            Raised when an invalid port number is passed.
        ``InvalidUseError``
            Raised when trying to open a MIDI port when a (virtual) port has
            already been opened by this instance.
        ``TypeError``
            Raised when an incompatible value type is passed for the ``port``
            or ``name`` parameter.

        """
        inout = self._check_port()

        if name is None:
            name = "RtMidi %s" % inout

        self.baseptr().openPort(port, _to_bytes(name))
        self._port = port
        return self

    def open_virtual_port(self, name=None):
        """Open a virtual MIDI input or output port.

        Only one port can be opened per ``MidiIn`` or ``MidiOut`` instance. An
        ``InvalidUseError`` exception is raised if an attempt is made to open a
        port on a ``MidiIn`` or ``MidiOut`` instance, which already opened a
        (virtual) port.

        A virtual port is not connected to a physical MIDI device or system
        port when first opened. You can connect it to another MIDI output with
        the OS-dependent tools provided by the low-level MIDI framework, e.g.
        ``aconnect`` for ALSA, ``jack_connect`` for JACK, or the Audio & MIDI
        settings dialog for CoreMIDI.

        .. note::
            Virtual ports are not supported by some backend APIs, namely the
            Windows MultiMedia API. You can use special MIDI drivers like `MIDI
            Yoke`_ or loopMIDI_ to provide hardware-independent virtual MIDI
            ports as an alternative.

        You can optionally pass a name for the RtMidi port with the ``name``
        keyword or the second positional argument. Names with non-ASCII
        characters in them have to be passed as unicode or UTF-8 encoded
        strings in Python 2. The default name is "RtMidi virtual input" resp.
        "RtMidi virtual output".

        .. note::
            Closing a port and opening it again with a different name does not
            change the port name. To change the port name, use the
            ``set_port_name`` method where supported, or delete its instance,
            create a new one and open the port again giving a different name.

            Also, to close a virtual input port, you have to delete its
            ``MidiIn`` or ``MidiOut`` instance.

        Exceptions:

        ``InvalidUseError``
            Raised when trying to open a virtual port when a (virtual) port has
            already been opened by this instance.
        ``TypeError``
            Raised when an incompatible value type is passed for the ``name``
            parameter.
        ``UnsupportedOperationError``
            Raised when trying to open a virtual MIDI port with the Windows
            MultiMedia API, which doesn't support virtual ports.

        .. _midi yoke: http://www.midiox.com/myoke.htm
        .. _loopmidi: http://www.tobias-erichsen.de/software/loopmidi.html

        """
        if self.get_current_api() == API_WINDOWS_MM:
            raise NotImplementedError("Virtual ports are not supported "
                                      "by the Windows MultiMedia API.")

        inout = self._check_port()
        self.baseptr().openVirtualPort(_to_bytes(("RtMidi virtual %s" % inout)
                                                if name is None else name))
        self._port = -1
        return self

    def close_port(self):
        """Close the MIDI input or output port opened via ``open_port``.

        It is safe to call this method repeatedly or if no port has been opened
        (yet) by this instance.

        Also cancels a callback function set with ``set_callback``.

        To close a virtual port opened via ``open_virtual_port``, you have to
        delete its ``MidiIn`` or ``MidiOut`` instance.

        """
        if self._port != -1:
            self._port = None
        self.baseptr().closePort()

    def set_client_name(self, name):
        """Set the name of the MIDI client.

        Names with non-ASCII characters in them have to be passed as unicode
        or UTF-8 encoded strings in Python 2.

        Currently only supported by the ALSA API backend.

        Exceptions:

        ``TypeError``
            Raised when an incompatible value type is passed for the ``name``
            parameter.
        ``UnsupportedOperationError``
            Raised when trying the backend API does not support changing the
            client name.

        """
        if self.get_current_api() in (API_MACOSX_CORE, API_UNIX_JACK, API_WINDOWS_MM):
            raise NotImplementedError(
                "API backend does not support changing the client name.")

        self.baseptr().setClientName(_to_bytes(name))

    def set_port_name(self, name):
        """Set the name of the currently opened port.

        Names with non-ASCII characters in them have to be passed as unicode
        or UTF-8 encoded strings in Python 2.

        Currently only supported by the ALSA and JACK API backends.

        Exceptions:

        ``InvalidUseError``
            Raised when no port is currently opened.
        ``TypeError``
            Raised when an incompatible value type is passed for the ``name``
            parameter.
        ``UnsupportedOperationError``
            Raised when trying the backend API does not support changing the
            port name.

        """
        if self.get_current_api() in (API_MACOSX_CORE, API_WINDOWS_MM):
            raise UnsupportedOperationError(
                "API backend does not support changing the port name.")

        if self._port is None:
            raise InvalidUseError("No port currently opened.")

        self.baseptr().setPortName(_to_bytes(name))

    def set_error_callback(self, func, data=None):
        """Register a callback function for errors.

        The callback function is called when an error occurs and must take
        three arguments. The first argument is a member of enum
        ``RtMidiError::Type``, represented by one of the ``ERRORTYPE_*``
        constants. The second argument is an error message. The third argument
        is the value of the ``data`` argument passed to this function when the
        callback is registered.

        .. note::
            A default error handler function is registered on new instances of
            ``MidiIn`` and ``MidiOut``, which turns errors reported by the C++
            layer into custom exceptions derived from ``RtMidiError``.

            If you replace this default error handler, be aware that the
            exception handling in your code probably needs to be adapted.

        Registering an error callback function replaces any previously
        registered error callback, including the above mentioned default error
        handler.

        """
        self._error_callback = (func, data, self._decode_string)
        self.baseptr().setErrorCallback(&_cb_error_func,
                                        <void *>self._error_callback)

    def cancel_error_callback(self):
        """Remove the registered callback function for errors.

        This can be safely called even when no callback function has been
        registered and reinstates the default error handler.

        """
        self.set_error_callback(_default_error_handler)


cdef class MidiIn(MidiBase):
    """Midi input client interface.

    ``rtmidi.MidiIn(rtapi=API_UNSPECIFIED, name="RtMidi Client", queue_size_limit=1024)``

    You can specify the low-level MIDI backend API to use via the ``rtapi``
    keyword or the first positional argument, passing one of the module-level
    ``API_*`` constants. You can get a list of compiled-in APIs with the
    module-level ``get_compiled_api`` function. If you pass ``API_UNSPECIFIED``
    (the default), the first compiled-in API, which has any input ports
    available, will be used.

    You can optionally pass a name for the MIDI client with the ``name``
    keyword or the second positional argument. Names with non-ASCII characters
    in them have to be passed as unicode or UTF-8 encoded strings in Python 2.
    The default name is ``"RtMidiIn Client"``.

    .. note::
        With some backend APIs (e.g. ALSA), the client name is set by the first
        ``MidiIn`` *or* ``MidiOut`` created by your program and does not change
        unless you either use the ``set_client_name`` method, or until *all*
        ``MidiIn`` and ``MidiOut`` instances are deleted and then a new one is
        created.

    The ``queue_size_limit`` argument specifies the size of the internal ring
    buffer in which incoming MIDI events are placed until retrieved via the
    ``get_message`` method (i.e. when no callback function is registered).
    The default value is ``1024`` (overriding the default value ``100`` of the
    underlying C++ RtMidi library).

    Exceptions:

    ``SystemError``
        Raised when the RtMidi backend initialization fails. The execption
        message should have more information on the cause of the error.

    ``TypeError``
        Raised when an incompatible value type is passed for a parameter.

    """

    cdef RtMidiIn *thisptr
    cdef object _callback

    cdef RtMidi* baseptr(self):
        return self.thisptr

    def __cinit__(self, Api rtapi=UNSPECIFIED, name=None,
                  unsigned int queue_size_limit=1024):
        """Create a new client instance for MIDI input.

        See the class docstring for a description of the constructor arguments.

        """
        if name is None:
            name = "RtMidiIn Client"

        try:
            self.thisptr = new RtMidiIn(rtapi, _to_bytes(name), queue_size_limit)
        except RuntimeError as exc:
            raise SystemError(str(exc), type=ERR_DRIVER_ERROR)

        self.set_error_callback(_default_error_handler)
        self._callback = None
        self._port = None
        self._deleted = False

    def get_current_api(self):
        """Return the low-level MIDI backend API used by this instance.

        Use this by comparing the returned value to the module-level ``API_*``
        constants, e.g.::

            midiin = rtmidi.MidiIn()

            if midiin.get_current_api() == rtmidi.API_UNIX_JACK:
                print("Using JACK API for MIDI input.")

        """
        return self.thisptr.getCurrentApi()

    def __dealloc__(self):
        """De-allocate pointer to C++ class instance."""
        if hasattr(self, "thisptr"):
            del self.thisptr

    def delete(self):
        """De-allocate pointer to C++ class instance.

        .. warning:: the instance **must not** be used anymore after calling
            this method, otherwise the program will crash with a segmentation
            fault!

            The reason this potentially dangerous method exists is that in
            some cases it is desirable to destroy the internal ``RtMidiIn``
            C++ class instance with immediate effect, thereby closing the
            backend MIDI API client and all the ports it opened. By merely
            using ``del`` on the ``rtmidi.MidiIn`` Python instance, the
            destruction of the C++ instance may be delayed for an arbitrary
            amount of time, until the Python garbage collector cleans up the
            instance.

        """
        if not self._deleted:
            del self.thisptr
            self._deleted = True

    @property
    def is_deleted(self):
        return self._deleted

    def cancel_callback(self):
        """Remove the registered callback function for MIDI input.

        This can be safely called even when no callback function has been
        registered.

        """
        if self._callback:
            self.thisptr.cancelCallback()
            self._callback = None

    def close_port(self):
        self.cancel_callback()
        MidiBase.close_port(self)

    close_port.__doc__ == MidiBase.close_port.__doc__

    def get_message(self):
        """Poll for MIDI input.

        Checks whether a MIDI event is available in the input buffer and
        returns a two-element tuple with the MIDI message and a delta time. The
        MIDI message is a list of integers representing the data bytes of the
        message, the delta time is a float representing the time in seconds
        elapsed since the reception of the previous MIDI event.

        The function does not block. When no MIDI message is available, it
        returns ``None``.

        """
        cdef vector[unsigned char] msg_v
        cdef double delta_time = self.thisptr.getMessage(&msg_v)

        if not msg_v.empty():
            message = [msg_v.at(i) for i in range(msg_v.size())]
            return (message, delta_time)

    def ignore_types(self, sysex=True, timing=True, active_sense=True):
        """Enable/Disable input filtering of certain types of MIDI events.

        By default System Exclusive (aka sysex), MIDI Clock and Active Sensing
        messages are filtered from the MIDI input and never reach your code,
        because they can fill up input buffers very quickly.

        To receive them, you can selectively disable the filtering of these
        event types.

        To enable reception - i.e. turn off the default filtering - of sysex
        messages, pass ``sysex = False``.

        To enable reception of MIDI Clock, pass ``timing = False``.

        To enable reception of Active Sensing, pass ``active_sense = False``.

        These arguments can of course be combined in one call, and they all
        default to ``True``.

        If you enable reception of any of these event types, be sure to either
        use an input callback function, which returns quickly or poll for MIDI
        input often. Otherwise you might lose MIDI input because the input
        buffer overflows.

        **Windows note:** the Windows Multi Media API uses fixed size buffers
        for the reception of sysex messages, whose number and size is set at
        compile time. Sysex messages longer than the buffer size can not be
        received properly when using the Windows Multi Media API.

        The default distribution of python-rtmidi sets the number of sysex
        buffers to four and the size of each to 8192 bytes. To change these
        values, edit the ``RT_SYSEX_BUFFER_COUNT`` and ``RT_SYSEX_BUFFER_SIZE``
        preprocessor defines in ``RtMidi.cpp`` and recompile.

        """
        self.thisptr.ignoreTypes(sysex, timing, active_sense)

    def set_callback(self, func, data=None):
        """Register a callback function for MIDI input.

        The callback function is called whenever a MIDI message is received and
        must take two arguments. The first argument is a two-element tuple with
        the MIDI message and a delta time, like the one returned by the
        ``get_message`` method. The second argument is value of the ``data``
        argument passed to ``set_callback`` when the callback is registered.
        The value of ``data`` can be any Python object. It can be used inside
        the callback function to access data that would not be in scope
        otherwise.

        Registering a callback function replaces any previously registered
        callback.

        The callback function is safely removed when the input port is closed
        or the ``MidiIn`` instance is deleted.

        """
        if self._callback:
            self.cancel_callback()

        self._callback = (func, data)
        self.thisptr.setCallback(&_cb_func, <void *>self._callback)


cdef class MidiOut(MidiBase):
    """Midi output client interface.

    ``rtmidi.MidiOut(rtapi=API_UNSPECIFIED, name="RtMidi Client")``

    You can specify the low-level MIDI backend API to use via the ``rtapi``
    keyword or the first positional argument, passing one of the module-level
    ``API_*`` constants. You can get a list of compiled-in APIs with the
    module-level ``get_compiled_api`` function. If you pass ``API_UNSPECIFIED``
    (the default), the first compiled-in API, which has any input ports
    available, will be used.

    You can optionally pass a name for the MIDI client with the ``name``
    keyword or the second positional argument. Names with non-ASCII characters
    in them have to be passed as unicode or UTF-8 encoded strings in Python 2.
    The default name is ``"RtMidiOut Client"``.

    .. note::
        With some backend APIs (e.g. ALSA), the client name is set by the first
        ``MidiIn`` *or* ``MidiOut`` created by your program and does not change
        unless you either use the ``set_client_name`` method, or until *all*
        ``MidiIn`` and ``MidiOut`` instances are deleted and then a new one is
        created.

    Exceptions:

    ``SystemError``
        Raised when the RtMidi backend initialization fails. The execption
        message should have more information on the cause of the error.

    ``TypeError``
        Raised when an incompatible value type is passed for a parameter.

    """

    cdef RtMidiOut *thisptr

    cdef RtMidi* baseptr(self):
        return self.thisptr

    def __cinit__(self, Api rtapi=UNSPECIFIED, name=None):
        """Create a new client instance for MIDI output.

        See the class docstring for a description of the constructor arguments.

        """
        if name is None:
            name = "RtMidiOut Client"

        try:
            self.thisptr = new RtMidiOut(rtapi, _to_bytes(name))
        except RuntimeError as exc:
            raise SystemError(str(exc), type=ERR_DRIVER_ERROR)

        self.set_error_callback(_default_error_handler)
        self._port = None
        self._deleted = False

    def __dealloc__(self):
        """De-allocate pointer to C++ class instance."""
        if hasattr(self, "thisptr"):
            del self.thisptr

    def delete(self):
        """De-allocate pointer to C++ class instance.

        .. warning:: the instance **must not** be used anymore after calling
            this method, otherwise the program will crash with a segmentation
            fault!

            The reason this potentially dangerous method exists is that in
            some cases it is desirable to destroy the internal ``RtMidiOut``
            C++ class instance with immediate effect, thereby closing the
            backend MIDI API client and all the ports it opened. By merely
            using ``del`` on the ``rtmidi.MidiOut`` Python instance, the
            destruction of the C++ instance may be delayed for an arbitrary
            amount of time, until the Python garbage collector cleans up the
            instance.

        """
        if not self._deleted:
            del self.thisptr
            self._deleted = True

    @property
    def is_deleted(self):
        return self._deleted

    def get_current_api(self):
        """Return the low-level MIDI backend API used by this instance.

        Use this by comparing the returned value to the module-level ``API_*``
        constants, e.g.::

            midiout = rtmidi.MidiOut()

            if midiout.get_current_api() == rtmidi.API_UNIX_JACK:
                print("Using JACK API for MIDI output.")

        """
        return self.thisptr.getCurrentApi()

    def send_message(self, message):
        """Send a MIDI message to the output port.

        The message must be passed as an iterable yielding integers, each
        element representing one byte of the MIDI message.

        Normal MIDI messages have a length of one to three bytes, but you can
        also send system exclusive messages, which can be arbitrarily long, via
        this method.

        No check is made whether the passed data constitutes a valid MIDI
        message but if it is longer than 3 bytes, the value of the first byte
        must be a start-of-sysex status byte, i.e. 0xF0.

        Exceptions:

        ``ValueError``
            Raised if ``message`` argument is empty or more than 3 bytes long
            and not a SysEx message.

        """
        cdef vector[unsigned char] msg_v

        if not message:
            raise ValueError("'message' must not be empty.")

        if len(message) > 3 and message[0] != 0xF0:
            raise ValueError("'message' longer than 3 bytes but does not "
                             "start with 0xF0.")

        for c in message:
            msg_v.push_back(c)

        self.thisptr.sendMessage(&msg_v)



#!/usr/bin/env python
#
# panic.py
#
"""Send AllSoundOff and ResetAllControllers on all JACK MIDI outputs and all channels."""

from __future__ import print_function

import time

import rtmidi
from rtmidi.midiconstants import (ALL_SOUND_OFF, CONTROL_CHANGE,
                                  RESET_ALL_CONTROLLERS)


midiout = rtmidi.MidiOut(rtapi=rtmidi.API_UNIX_JACK)
print(__doc__)

for portnum, portname in enumerate(midiout.get_ports()):
    print("Port:", portname)

    with midiout.open_port(portnum):
        for channel in range(16):
            midiout.send_message([CONTROL_CHANGE, ALL_SOUND_OFF, 0])
            midiout.send_message([CONTROL_CHANGE, RESET_ALL_CONTROLLERS, 0])
            time.sleep(0.05)

        time.sleep(0.1)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Tests for the error conditions"""

import unittest

import rtmidi
from rtmidi import InvalidPortError, InvalidUseError


class TestErrors(unittest.TestCase):

    INVALID_PORT_NUMBER = 9999
    INVALID_NAME_TYPES = (0, 666, 3.141, object)

    def setUp(self):
        self.midi_out = rtmidi.MidiOut()
        self.midi_in = rtmidi.MidiIn()

    def test_midiout_invalid_client_name_type(self):
        for name in self.INVALID_NAME_TYPES:
            with self.assertRaises(TypeError):
                rtmidi.MidiOut(rtapi=rtmidi.API_RTMIDI_DUMMY, name=name)

    def test_midiin_invalid_client_name_type(self):
        for name in self.INVALID_NAME_TYPES:
            with self.assertRaises(TypeError):
                rtmidi.MidiIn(rtapi=rtmidi.API_RTMIDI_DUMMY, name=name)

    def test_midiout_open_invalid_port(self):
        with self.assertRaises(InvalidPortError):
            self.midi_out.open_port(self.INVALID_PORT_NUMBER)

    def test_midiin_open_invalid_port(self):
        with self.assertRaises(InvalidPortError):
            self.midi_in.open_port(self.INVALID_PORT_NUMBER)

    def test_midiout_open_invalid_port_name_type(self):
        for name in self.INVALID_NAME_TYPES:
            with self.assertRaises(TypeError):
                self.midi_out.open_port(name=name)

    def test_midiin_open_invalid_port_name_type(self):
        for name in self.INVALID_NAME_TYPES:
            with self.assertRaises(TypeError):
                self.midi_in.open_port(name=name)

    def test_midiout_double_open_port(self):
        self.midi_out.open_port()
        with self.assertRaises(InvalidUseError):
            self.midi_out.open_port()

    def test_midiin_double_open_port(self):
        self.midi_in.open_port()
        with self.assertRaises(InvalidUseError):
            self.midi_in.open_port()

    def test_midiout_double_open_virtual_port(self):
        self.midi_out.open_virtual_port()
        with self.assertRaises(InvalidUseError):
            self.midi_out.open_virtual_port()

    def test_midiin_double_open_virtual_port(self):
        self.midi_in.open_virtual_port()
        with self.assertRaises(InvalidUseError):
            self.midi_in.open_virtual_port()


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Custom distutils command to fill placeholders in text files with release
meta-data.

"""

from os.path import join
from string import Template

try:
    basestring  # noqa
except:
    basestring = str

from distutils.core import Command
from distutils.dist import DistributionMetadata
from distutils.log import error, info
from distutils.util import split_quoted


DistributionMetadata.templates = None


class FillTemplate(Command):
    """Custom distutils command to fill text templates with release meta data.
    """

    description = "Fill placeholders in documentation text file templates"

    user_options = [
        ('templates=', None, "Template text files to fill")
    ]

    def initialize_options(self):
        self.templates = ''
        self.template_ext = '.in'

    def finalize_options(self):
        if isinstance(self.templates, basestring):
            self.templates = split_quoted(self.templates)

        self.templates += getattr(self.distribution.metadata, 'templates', None) or []

        for tmpl in self.templates:
            if not tmpl.endswith(self.template_ext):
                raise ValueError("Template file '%s' does not have expected "
                                 "extension '%s'." % (tmpl, self.template_ext))

    def run(self):
        metadata = self.get_metadata()

        for infilename in self.templates:
            try:
                info("Reading template '%s'...", infilename)
                with open(infilename) as infile:
                    tmpl = Template(infile.read())
                    outfilename = infilename.rstrip(self.template_ext)

                    info("Writing filled template to '%s'.", outfilename)
                    with open(outfilename, 'w') as outfile:
                        outfile.write(tmpl.safe_substitute(metadata))
            except:
                error("Could not open template '%s'.", infilename)

    def get_metadata(self):
        data = dict()
        for attr in self.distribution.metadata.__dict__:
            if not callable(attr):
                data[attr] = getattr(self.distribution.metadata, attr)

        data['cpp_info'] = open(join("src", '_rtmidi.cpp')).readline().strip()
        return data



#!/usr/bin/env python
#
# midi2command.py
#
"""Execute external commands when specific MIDI messages are received.

Example configuration (in YAML syntax)::

    - name: My Backingtracks
      description: Play audio file with filename matching <data1>-playback.mp3
        when program change on channel 16 is received
      status: programchange
      channel: 16
      command: plaympeg %(data1)03i-playback.mp3
    - name: My Lead Sheets
      description: Open PDF with filename matching <data2>-sheet.pdf
        when control change 14 on channel 16 is received
      status: controllerchange
      channel: 16
      data: 14
      command: evince %(data2)03i-sheet.pdf

"""

import argparse
import logging
import shlex
import subprocess
import sys
import time

from os.path import exists

try:
    from functools import lru_cache
except ImportError:
    # Python < 3.2
    try:
        from backports.functools_lru_cache import lru_cache
    except ImportError:
        lru_cache = lambda: lambda func: func

import yaml

import rtmidi
from rtmidi.midiutil import open_midiinput
from rtmidi.midiconstants import (CHANNEL_PRESSURE, CONTROLLER_CHANGE, NOTE_ON, NOTE_OFF,
                                  PITCH_BEND, POLY_PRESSURE, PROGRAM_CHANGE)


log = logging.getLogger('midi2command')
BACKEND_MAP = {
    'alsa': rtmidi.API_LINUX_ALSA,
    'jack': rtmidi.API_UNIX_JACK,
    'coremidi': rtmidi.API_MACOSX_CORE,
    'windowsmm': rtmidi.API_WINDOWS_MM
}
STATUS_MAP = {
    'noteon': NOTE_ON,
    'noteoff': NOTE_OFF,
    'programchange': PROGRAM_CHANGE,
    'controllerchange': CONTROLLER_CHANGE,
    'pitchbend': PITCH_BEND,
    'polypressure': POLY_PRESSURE,
    'channelpressure': CHANNEL_PRESSURE
}


class Command(object):
    def __init__(self, name='', description='', status=0xB0, channel=None, data=None,
                 command=None):
        self.name = name
        self.description = description
        self.status = status
        self.channel = channel
        self.command = command

        if data is None or isinstance(data, int):
            self.data = data
        elif hasattr(data, 'split'):
            self.data = map(int, data.split())
        else:
            raise TypeError("Could not parse 'data' field.")


class MidiInputHandler(object):
    def __init__(self, port, config):
        self.port = port
        self._wallclock = time.time()
        self.commands = dict()
        self.load_config(config)

    def __call__(self, event, data=None):
        event, deltatime = event
        self._wallclock += deltatime

        if event[0] < 0xF0:
            channel = (event[0] & 0xF) + 1
            status = event[0] & 0xF0
        else:
            status = event[0]
            channel = None

        data1 = data2 = None
        num_bytes = len(event)

        if num_bytes >= 2:
            data1 = event[1]
        if num_bytes >= 3:
            data2 = event[2]

        log.debug("[%s] @%i CH:%2s %02X %s %s", self.port, self._wallclock,
                  channel or '-', status, data1, data2 or '')

        # Look for matching command definitions
        cmd = self.lookup_command(status, channel, data1, data2)

        if cmd:
            cmdline = cmd.command % dict(
                channel=channel,
                data1=data1,
                data2=data2,
                status=status)
            self.do_command(cmdline)

    @lru_cache()
    def lookup_command(self, status, channel, data1, data2):
        for cmd in self.commands.get(status, []):
            if channel is not None and cmd.channel != channel:
                continue

            if (data1 is None and data2 is None) or cmd.data is None:
                return cmd
            elif isinstance(cmd.data, int) and cmd.data == data1:
                return cmd
            elif (isinstance(cmd.data, (list, tuple)) and
                    cmd.data[0] == data1 and cmd.data[1] == data2):
                return cmd

    def do_command(self, cmdline):
        log.info("Calling external command: %s", cmdline)
        try:
            args = shlex.split(cmdline)
            subprocess.Popen(args)
        except:  # noqa: E722
            log.exception("Error calling external command.")

    def load_config(self, filename):
        if not exists(filename):
            raise IOError("Config file not found: %s" % filename)

        with open(filename) as patch:
            data = yaml.load(patch)

        for cmdspec in data:
            try:
                if isinstance(cmdspec, dict) and 'command' in cmdspec:
                    cmd = Command(**cmdspec)
                elif len(cmdspec) >= 2:
                    cmd = Command(*cmdspec)
            except (TypeError, ValueError) as exc:
                log.debug(cmdspec)
                raise IOError("Invalid command specification: %s" % exc)
            else:
                status = STATUS_MAP.get(cmd.status.strip().lower())

                if status is None:
                    try:
                        int(cmd.status)
                    except:  # noqa: E722
                        log.error("Unknown status '%s'. Ignoring command",
                                  cmd.status)

                log.debug("Config: %s\n%s\n", cmd.name, cmd.description)
                self.commands.setdefault(status, []).append(cmd)


def main(args=None):
    """Main program function.

    Parses command line (parsed via ``args`` or from ``sys.argv``), detects
    and optionally lists MIDI input ports, opens given MIDI input port,
    and attaches MIDI input handler object.

    """
    parser = argparse.ArgumentParser(description=__doc__.splitlines()[0])
    padd = parser.add_argument
    padd('-b', '--backend', choices=sorted(BACKEND_MAP),
         help='MIDI backend API (default: OS dependent)')
    padd('-p', '--port',
         help='MIDI input port name or number (default: open virtual input)')
    padd('-v', '--verbose',
         action="store_true", help='verbose output')
    padd(dest='config', metavar="CONFIG",
         help='Configuration file in YAML syntax.')

    args = parser.parse_args(args)

    logging.basicConfig(format="%(name)s: %(levelname)s - %(message)s",
                        level=logging.DEBUG if args.verbose else logging.INFO)

    try:
        midiin, port_name = open_midiinput(
            args.port,
            use_virtual=True,
            api=BACKEND_MAP.get(args.backend, rtmidi.API_UNSPECIFIED),
            client_name='midi2command',
            port_name='MIDI input')
    except (IOError, ValueError) as exc:
        return "Could not open MIDI input: %s" % exc
    except (EOFError, KeyboardInterrupt):
        return

    log.debug("Attaching MIDI input callback handler.")
    midiin.set_callback(MidiInputHandler(port_name, args.config))

    log.info("Entering main loop. Press Control-C to exit.")
    try:
        # just wait for keyboard interrupt in main thread
        while True:
            time.sleep(1)
    except KeyboardInterrupt:
        print('')
    finally:
        midiin.close_port()
        del midiin


if __name__ == '__main__':
    sys.exit(main() or 0)



# -*- coding:utf-8 -*-
from __future__ import absolute_import
from .version import version as __version__  # noqa
from ._rtmidi import *  # noqa
from ._rtmidi import __doc__  # noqa
del absolute_import



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# sysexsaver.py
#
"""Save all revceived sysex messages to given directory."""

import argparse
import logging
import os
import re
import sys
import time

from datetime import datetime
from os.path import exists, join

from rtmidi.midiconstants import END_OF_EXCLUSIVE, SYSTEM_EXCLUSIVE
from rtmidi.midiutil import open_midiinput

from .manufacturers import manufacturers
from .models import models


log = logging.getLogger('sysexsaver')


def sanitize_name(s, replace='/?*&\\'):
    s = s.strip()
    s = re.sub(r'\s+', '_', s)
    for c in replace:
        s = s.replace(c, '_')
    return s.lower()


class SysexMessage(object):
    @classmethod
    def fromdata(cls, data):
        self = cls()
        if data[0] != SYSTEM_EXCLUSIVE:
            raise ValueError("Message does not start with 0xF0", data)
        if data[-1] != END_OF_EXCLUSIVE:
            raise ValueError("Message does not end with 0xF7", data)
        if len(data) < 5:
            raise ValueError("Message too short", data)
        if data[1] == 0:
            self.manufacturer_id = (data[1], data[2], data[3])
            self.model_id = data[5]
            self.device_id = data[6]
        else:
            self.manufacturer_id = data[1]
            self.model_id = data[2]
            self.device_id = data[3]

        self._data = data
        return self

    def __getitem__(self, i):
        return self._data[i]

    def __getslice(self, i, j):
        return self._data[i:j]

    @property
    def manufacturer(self):
        mname = manufacturers.get(self.manufacturer_id)
        if mname:
            return mname[0]

    @property
    def manufacturer_tag(self):
        mname = manufacturers.get(self.manufacturer_id, [])
        if len(mname) >= 2:
            return mname[1]
        elif mname:
            return mname[0]

    @property
    def model(self):
        model_name = models.get(self.manufacturer_id, {}).get(self.model_id)
        return model_name[0] if model_name else "0x%02X" % self.model_id

    @property
    def model_tag(self):
        model_name = models.get(self.manufacturer_id, {}).get(self.model_id, [])
        if len(model_name) >= 2:
            return model_name[1]
        elif model_name:
            return model_name[1]
        else:
            "0x%02X" % self.model_id

    def __repr__(self):
        return "".join(["%02X " % b for b in self._data])

    def as_bytes(self):
        if bytes == str:
            return "".join([chr(b) for b in self._data])
        else:
            return bytes(self._data)


class SysexSaver(object):
    """MIDI input callback handler object."""

    fn_tmpl = "%(manufacturer)s-%(device)s-%(timestamp)s.syx"
    fn_named_tmpl = "%(manufacturer)s-%(device)s-%(name)s-%(timestamp)s.syx"

    def __init__(self, portname, directory, debug=False):
        self.portname = portname
        self.directory = directory
        self.debug = debug

    def __call__(self, event, data=None):
        try:
            message, deltatime = event
            if message[:1] != [SYSTEM_EXCLUSIVE]:
                return

            dt = datetime.now()
            log.debug("[%s: %s] Received sysex msg of %i bytes." % (
                self.portname, dt.strftime('%x %X'), len(message)))
            sysex = SysexMessage.fromdata(message)

            # XXX: This should be implemented in a subclass
            #      loaded via a plugin infrastructure
            data = dict(timestamp=dt.strftime('%Y%m%dT%H%M%S.%f'))
            data['manufacturer'] = sanitize_name(
                sysex.manufacturer_tag or 'unknown')
            data['device'] = sanitize_name(sysex.model_tag or 'unknown')

            if sysex.manufacturer_id == 62 and sysex.model_id == 0x0E:
                if sysex[4] == 0x10:
                    # sound dump
                    name = "".join(chr(c) for c in sysex[247:263]).rstrip('_')
                elif sysex[4] == 0x11:
                    # multi dump
                    name = "".join(chr(c) for c in sysex[23:38]).rstrip('_')
                elif sysex[4] == 0x12:
                    # wave dump
                    if sysex[5] > 1:
                        name = "userwave_%04i"
                    else:
                        name = "romwave_%03i"
                    name = name % ((sysex[5] << 7) | sysex[6])
                elif sysex[4] == 0x13:
                    # wave table dump
                    if sysex[6] >= 96:
                        name = "userwavetable_%03i" % (sysex[6] + 1)
                    else:
                        name = "romwavetable_%03i" % (sysex[6] + 1)
                else:
                    name = "%02X" % sysex[4]

                data['name'] = sanitize_name(name)

            outfn = join(self.directory, (
                self.fn_named_tmpl if 'name' in data else self.fn_tmpl) % data)

            if exists(outfn):
                log.error("Output file already exists, will not overwrite.")
            else:
                data = sysex.as_bytes()
                with open(outfn, 'wb') as outfile:
                    outfile.write(data)
                    log.info("Sysex message of %i bytes written to '%s'.",
                             len(data), outfn)
        except Exception as exc:
            msg = "Error handling MIDI message: %s" % exc.args[0]
            if self.debug:
                log.debug(msg)
                if len(exc.args) >= 2:
                    log.debug("Message data: %r", exc.args[1])
            else:
                log.error(msg)


def main(args=None):
    """Save revceived sysex message to directory given on command line."""
    parser = argparse.ArgumentParser(description=__doc__)
    padd = parser.add_argument
    padd('-o', '--outdir', default=os.getcwd(),
         help="Output directory (default: current working directory).")
    padd('-p', '--port',
         help='MIDI output port number (default: ask)')
    padd('-v', '--verbose', action="store_true",
         help='verbose output')

    args = parser.parse_args(args)

    logging.basicConfig(format="%(name)s: %(levelname)s - %(message)s",
                        level=logging.DEBUG if args.verbose else logging.INFO)

    try:
        midiin, port = open_midiinput(args.port)
    except IOError as exc:
        log.error(exc)
        return 1
    except (EOFError, KeyboardInterrupt):
        return 0

    ss = SysexSaver(port, args.outdir, args.verbose)

    log.debug("Attaching MIDI input callback handler.")
    midiin.set_callback(ss)
    log.debug("Enabling reception of sysex messages.")
    midiin.ignore_types(sysex=False)

    log.info("Waiting for sysex reception. Press Control-C to exit.")
    try:
        # just wait for keyboard interrupt in main thread
        while True:
            time.sleep(1)
    except KeyboardInterrupt:
        print('')
    finally:
        log.debug("Exit.")
        midiin.close_port()
        del midiin


if __name__ == '__main__':
    sys.exit(main() or 0)



# -*- coding: utf-8 -*-
"""Map manufacturer's model ID to name."""

models = {
    # id: (name, shortname),
    1: {},  # Sequential Circuits
    4: {},  # Moog
    6: {},  # Lexicon
    7: {},  # Kurzweil
    15: {},  # Ensoniq
    16: {},  # Oberheim
    17: {},  # Apple
    24: {},  # Emu
    26: {},  # ART
    34: {},  # Synthaxe
    36: {},  # Hohner
    41: {},  # PPG
    43: {},  # SSL
    47: {},  # Elka / General Music
    48: {},  # Dynacord
    51: {},  # Clavia (Nord)
    54: {},  # Cheetah
    # Waldorf Electronics GmbH
    62: {
        0x0E: ('microWAVE II/XT(k)', 'microwave2'),
    },
    64: {},  # Kawai Musical Instruments MFG. CO. Ltd
    65: {},  # Roland Corporation
    66: {},  # Korg Inc.
    67: {},  # Yamaha Corporation
    68: {},  # Casio Computer Co. Ltd
    70: {},  # Kamiya Studio Co. Ltd
    71: {},  # Akai Electric Co. Ltd.
    72: {},  # Victor Company of Japan, Ltd.
    75: {},  # Fujitsu Limited
    76: {},  # Sony Corporation
    78: {},  # Teac Corporation
    80: {},  # Matsushita Electric Industrial Co. , Ltd
    81: {},  # Fostex Corporation
    82: {},  # Zoom Corporation
    84: {},  # Matsushita Communication Industrial Co., Ltd.
    85: {},  # Suzuki Musical Instruments MFG. Co., Ltd.
    86: {},  # Fuji Sound Corporation Ltd.
    87: {},  # Acoustic Technical Laboratory, Inc.
    89: {},  # Faith, Inc.
    90: {},  # Internet Corporation
    92: {},  # Seekers Co. Ltd.
    95: {},  # SD Card Association
    0x7D: {},  # Non-commercial
    0x7E: {},  # Universal Non-Realtime
    0x7F: {},  # Universal Realtime
    (0, 0, 7): {},  # Digital Music Corporation
    (0, 0, 14): {},  # Alesis
    (0, 0, 21): {},  # KAT
    (0, 0, 22): {},  # Opcode
    (0, 0, 26): {},  # Allen & Heath Brenell
    (0, 0, 27): {},  # Peavey Electronics
    (0, 0, 28): {},  # 360 Systems
    (0, 0, 32): {},  # Axxes
    (0, 0, 116): {},  # Ta Horng Musical Instrument
    (0, 0, 117): {},  # e-Tek Labs (Forte Tech)
    (0, 0, 118): {},  # Electro-Voice
    (0, 0, 119): {},  # Midisoft Corporation
    (0, 0, 120): {},  # QSound Labs
    (0, 0, 121): {},  # Westrex
    (0, 0, 122): {},  # Nvidia
    (0, 0, 123): {},  # ESS Technology
    (0, 0, 124): {},  # Media Trix Peripherals
    (0, 0, 125): {},  # Brooktree Corp
    (0, 0, 126): {},  # Otari Corp
    (0, 0, 127): {},  # Key Electronics, Inc.
    (0, 1, 0): {},  # Shure Incorporated
    (0, 1, 1): {},  # AuraSound
    (0, 1, 2): {},  # Crystal Semiconductor
    (0, 1, 3): {},  # Conexant (Rockwell)
    (0, 1, 4): {},  # Silicon Graphics
    (0, 1, 5): {},  # M-Audio (Midiman)
    (0, 1, 6): {},  # PreSonus
    (0, 1, 8): {},  # Topaz Enterprises
    (0, 1, 9): {},  # Cast Lighting
    (0, 1, 10): {},  # Microsoft
    (0, 1, 11): {},  # Sonic Foundry
    (0, 1, 12): {},  # Line 6 (Fast Forward)
    (0, 1, 13): {},  # Beatnik Inc
    (0, 1, 14): {},  # Van Koevering Company
    (0, 1, 15): {},  # Altech Systems
    (0, 1, 16): {},  # S & S Research
    (0, 1, 17): {},  # VLSI Technology
    (0, 1, 18): {},  # Chromatic Research
    (0, 1, 19): {},  # Sapphire
    (0, 1, 20): {},  # IDRC
    (0, 1, 21): {},  # Justonic Tuning
    (0, 1, 22): {},  # TorComp Research Inc.
    (0, 1, 23): {},  # Newtek Inc.
    (0, 1, 24): {},  # Sound Sculpture
    (0, 1, 25): {},  # Walker Technical
    (0, 1, 26): {},  # Digital Harmony (PAVO)
    (0, 1, 27): {},  # InVision Interactive
    (0, 1, 28): {},  # T-Square Design
    (0, 1, 29): {},  # Nemesys Music Technology
    (0, 1, 30): {},  # DBX Professional (Harman Intl)
    (0, 1, 31): {},  # Syndyne Corporation
    (0, 1, 32): {},  # Bitheadz
    (0, 1, 33): {},  # Cakewalk Music Software
    (0, 1, 34): {},  # Analog Devices
    (0, 1, 35): {},  # National Semiconductor
    (0, 1, 36): {},  # Boom Theory / Adinolfi Alternative Percussion
    (0, 1, 37): {},  # Virtual DSP Corporation
    (0, 1, 38): {},  # Antares Systems
    (0, 1, 39): {},  # Angel Software
    (0, 1, 40): {},  # St Louis Music
    (0, 1, 41): {},  # Lyrrus dba G-VOX
    (0, 1, 42): {},  # Ashley Audio Inc.
    (0, 1, 43): {},  # Vari-Lite Inc.
    (0, 1, 44): {},  # Summit Audio Inc.
    (0, 1, 45): {},  # Aureal Semiconductor Inc.
    (0, 1, 46): {},  # SeaSound LLC
    (0, 1, 47): {},  # U.S. Robotics
    (0, 1, 48): {},  # Aurisis Research
    (0, 1, 49): {},  # Nearfield Research
    (0, 1, 50): {},  # FM7 Inc
    (0, 1, 51): {},  # Swivel Systems
    (0, 1, 52): {},  # Hyperactive Audio Systems
    (0, 1, 53): {},  # MidiLite (Castle Studios Productions)
    (0, 1, 54): {},  # Radikal Technologies
    (0, 1, 55): {},  # Roger Linn Design
    (0, 1, 56): {},  # TC-Helicon Vocal Technologies
    (0, 1, 57): {},  # Event Electronics
    (0, 1, 58): {},  # Sonic Network Inc
    (0, 1, 59): {},  # Realtime Music Solutions
    (0, 1, 60): {},  # Apogee Digital
    (0, 1, 61): {},  # Classical Organs, Inc.
    (0, 1, 62): {},  # Microtools Inc.
    (0, 1, 63): {},  # Numark Industries
    (0, 1, 64): {},  # Frontier Design Group, LLC
    (0, 1, 65): {},  # Recordare LLC
    (0, 1, 66): {},  # Starr Labs
    (0, 1, 67): {},  # Voyager Sound Inc.
    (0, 1, 68): {},  # Manifold Labs
    (0, 1, 69): {},  # Aviom Inc.
    (0, 1, 70): {},  # Mixmeister Technology
    (0, 1, 71): {},  # Notation Software
    (0, 1, 72): {},  # Mercurial Communications
    (0, 1, 73): {},  # Wave Arts
    (0, 1, 74): {},  # Logic Sequencing Devices
    (0, 1, 75): {},  # Axess Electronics
    (0, 1, 76): {},  # Muse Research
    (0, 1, 77): {},  # Open Labs
    (0, 1, 78): {},  # Guillemot R&D Inc
    (0, 1, 79): {},  # Samson Technologies
    (0, 1, 80): {},  # Electronic Theatre Controls
    (0, 1, 81): {},  # Blackberry (RIM)
    (0, 1, 82): {},  # Mobileer
    (0, 1, 83): {},  # Synthogy
    (0, 1, 84): {},  # Lynx Studio Technology Inc.
    (0, 1, 85): {},  # Damage Control Engineering LLC
    (0, 1, 86): {},  # Yost Engineering, Inc.
    (0, 1, 87): {},  # Brooks & Forsman Designs LLC / DrumLite
    (0, 1, 88): {},  # Infinite Response
    (0, 1, 89): {},  # Garritan Corp
    (0, 1, 90): {},  # Plogue Art et Technologie, Inc
    (0, 1, 91): {},  # RJM Music Technology
    (0, 1, 92): {},  # Custom Solutions Software
    (0, 1, 93): {},  # Sonarcana LLC
    (0, 1, 94): {},  # Centrance
    (0, 1, 95): {},  # Kesumo LLC
    (0, 1, 96): {},  # Stanton (Gibson)
    (0, 1, 97): {},  # Livid Instruments
    (0, 1, 98): {},  # First Act / 745 Media
    (0, 1, 99): {},  # Pygraphics, Inc.
    (0, 1, 100): {},  # Panadigm Innovations Ltd
    (0, 1, 101): {},  # Avedis Zildjian Co
    (0, 1, 102): {},  # Auvital Music Corp
    (0, 1, 103): {},  # Inspired Instruments Inc
    (0, 1, 104): {},  # Chris Grigg Designs
    (0, 1, 105): {},  # Slate Digital LLC
    (0, 1, 106): {},  # Mixware
    (0, 1, 107): {},  # Social Entropy
    (0, 1, 108): {},  # Source Audio LLC
    (0, 1, 109): {},  # Ernie Ball / Music Man
    (0, 1, 110): {},  # Fishman Transducers
    (0, 1, 111): {},  # Custom Audio Electronics
    (0, 1, 112): {},  # American Audio/DJ
    (0, 1, 113): {},  # Mega Control Systems
    (0, 1, 114): {},  # Kilpatrick Audio
    (0, 1, 115): {},  # iConnectivity
    (0, 1, 116): {},  # Fractal Audio
    (0, 1, 117): {},  # NetLogic Microsystems
    (0, 1, 118): {},  # Music Computing
    (0, 1, 119): {},  # Nektar Technology Inc
    (0, 1, 120): {},  # Zenph Sound Innovations
    (0, 1, 121): {},  # DJTechTools.com
    (0, 1, 122): {},  # Rezonance Labs
    (0, 1, 123): {},  # Decibel Eleven
    (0, 1, 124): {},  # CNMAT
    (0, 1, 125): {},  # Media Overkill
    (0, 1, 126): {},  # Confusionists LLC
    (0, 32, 39): {},  # Acorn Computer
    (0, 32, 41): {},  # Focusrite/Novation
    (0, 32, 42): {},  # Samkyung Mechatronics
    (0, 32, 43): {},  # Medeli Electronics Co.
    (0, 32, 44): {},  # Charlie Lab SRL
    (0, 32, 45): {},  # Blue Chip Music Technology
    (0, 32, 46): {},  # BEE OH Corp
    (0, 32, 47): {},  # LG Semicon America
    (0, 32, 48): {},  # TESI
    (0, 32, 49): {},  # EMAGIC
    (0, 32, 50): {},  # Behringer GmbH
    (0, 32, 51): {},  # Access Music Electronics
    (0, 32, 52): {},  # Synoptic
    (0, 32, 53): {},  # Hanmesoft
    (0, 32, 54): {},  # Terratec Electronic GmbH
    (0, 32, 55): {},  # Proel SpA
    (0, 32, 56): {},  # IBK MIDI
    (0, 32, 57): {},  # IRCAM
    (0, 32, 58): {},  # Propellerhead Software
    (0, 32, 59): {},  # Red Sound Systems Ltd
    (0, 32, 60): {},  # Elektron ESI AB
    (0, 32, 61): {},  # Sintefex Audio
    (0, 32, 62): {},  # MAM (Music and More)
    (0, 32, 63): {},  # Amsaro GmbH
    (0, 32, 64): {},  # CDS Advanced Technology BV
    (0, 32, 65): {},  # Touched By Sound GmbH
    (0, 32, 66): {},  # DSP Arts
    (0, 32, 67): {},  # Phil Rees Music Tech
    (0, 32, 68): {},  # Stamer Musikanlagen GmbH
    (0, 32, 69): {},  # Musical Muntaner S.A. dba Soundart
    (0, 32, 70): {},  # C-Mexx Software
    (0, 32, 71): {},  # Klavis Technologies
    (0, 32, 72): {},  # Noteheads AB
    (0, 32, 73): {},  # Algorithmix
    (0, 32, 74): {},  # Skrydstrup R&D
    (0, 32, 75): {},  # Professional Audio Company
    (0, 32, 76): {},  # NewWave Labs (MadWaves)
    (0, 32, 77): {},  # Vermona
    (0, 32, 78): {},  # Nokia
    (0, 32, 79): {},  # Wave Idea
    (0, 32, 80): {},  # Hartmann GmbH
    (0, 32, 81): {},  # Lion's Tracs
    (0, 32, 82): {},  # Analogue Systems
    (0, 32, 83): {},  # Focal-JMlab
    (0, 32, 84): {},  # Ringway Electronics (Chang-Zhou) Co Ltd
    (0, 32, 85): {},  # Faith Technologies (Digiplug)
    (0, 32, 86): {},  # Showworks
    (0, 32, 87): {},  # Manikin Electronic
    (0, 32, 88): {},  # 1 Come Tech
    (0, 32, 89): {},  # Phonic Corp
    (0, 32, 90): {},  # Dolby Australia (Lake)
    (0, 32, 91): {},  # Silansys Technologies
    (0, 32, 92): {},  # Winbond Electronics
    (0, 32, 93): {},  # Cinetix Medien und Interface GmbH
    (0, 32, 94): {},  # A&G Soluzioni Digitali
    (0, 32, 95): {},  # Sequentix Music Systems
    (0, 32, 96): {},  # Oram Pro Audio
    (0, 32, 97): {},  # Be4 Ltd
    (0, 32, 98): {},  # Infection Music
    (0, 32, 99): {},  # Central Music Co. (CME)
    (0, 32, 100): {},  # genoQs Machines GmbH
    (0, 32, 101): {},  # Medialon
    (0, 32, 102): {},  # Waves Audio Ltd
    (0, 32, 103): {},  # Jerash Labs
    (0, 32, 104): {},  # Da Fact
    (0, 32, 105): {},  # Elby Designs
    (0, 32, 106): {},  # Spectral Audio
    (0, 32, 107): {},  # Arturia
    (0, 32, 108): {},  # Vixid
    (0, 32, 109): {},  # C-Thru Music
    (0, 32, 110): {},  # Ya Horng Electronic Co LTD
    (0, 32, 111): {},  # SM Pro Audio
    (0, 32, 112): {},  # OTO MACHINES
    (0, 32, 113): {},  # ELZAB S.A., G LAB
    (0, 32, 114): {},  # Blackstar Amplification Ltd
    (0, 32, 115): {},  # M3i Technologies GmbH
    (0, 32, 116): {},  # Gemalto (from Xiring)
    (0, 32, 117): {},  # Prostage SL
    (0, 32, 118): {},  # Teenage Engineering
    (0, 32, 119): {},  # Tobias Erichsen Consulting
    (0, 32, 120): {},  # Nixer Ltd
    (0, 32, 121): {},  # Hanpin Electron Co Ltd
    (0, 32, 122): {},  # "MIDI-hardware" R.Sowa
    (0, 32, 123): {},  # Beyond Music Industrial Ltd
    (0, 32, 124): {},  # Kiss Box B.V.
    (0, 32, 125): {},  # Misa Digital Technologies Ltd
    (0, 32, 126): {},  # AI Musics Technology Inc
    (0, 32, 127): {},  # Serato Inc LP
    (0, 33, 0): {},  # Limex Music Handles GmbH
    (0, 33, 1): {},  # Kyodday/Tokai
    (0, 33, 2): {
        0x02: ("Shruthi-1", "shruthi-1"),
    },  # Mutable Instruments
    (0, 33, 3): {},  # PreSonus Software Ltd
    (0, 33, 4): {},  # Xiring
    (0, 33, 5): {},  # Fairlight Instruments Pty Ltd
    (0, 33, 6): {},  # Musicom Lab
    (0, 33, 7): {},  # VacoLoco
    (0, 33, 8): {},  # RWA (Hong Kong) Limited
    (0, 33, 9): {},  # Native Instruments
    (0, 33, 10): {},  # Naonext
    (0, 33, 11): {},  # MFB
    (0, 33, 12): {},  # Teknel Research
    (0, 33, 13): {},  # Ploytec GmbH
    (0, 33, 14): {},  # Surfin Kangaroo Studio
    (0, 33, 15): {},  # Philips Electronics HK Ltd
    (0, 33, 16): {},  # ROLI Ltd
    (0, 64, 0): {},  # Crimson Technology Inc.
    (0, 64, 1): {},  # Softbank Mobile Corp
    (0, 64, 3): {},  # D&M Holdings Inc.
}



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# midifilter/__main__.py
#
"""Simple MIDI filter / processor."""

from __future__ import absolute_import

import argparse
import logging
import sys
import threading
import time

try:
    import Queue as queue
except ImportError:  # Python 3
    import queue

from rtmidi.midiutil import open_midiport

from .filters import MapControllerValue, MonoPressureToCC, Transpose


log = logging.getLogger("midifilter")


class MidiDispatcher(threading.Thread):
    def __init__(self, midiin, midiout, *filters):
        super(MidiDispatcher, self).__init__()
        self.midiin = midiin
        self.midiout = midiout
        self.filters = filters
        self._wallclock = time.time()
        self.queue = queue.Queue()

    def __call__(self, event, data=None):
        message, deltatime = event
        self._wallclock += deltatime
        log.debug("IN: @%0.6f %r", self._wallclock, message)
        self.queue.put((message, self._wallclock))

    def run(self):
        log.debug("Attaching MIDI input callback handler.")
        self.midiin.set_callback(self)

        while True:
            event = self.queue.get()

            if event is None:
                break

            events = [event]

            for filter_ in self.filters:
                events = list(filter_.process(events))

            for event in events:
                log.debug("Out: @%0.6f %r", event[1], event[0])
                self.midiout.send_message(event[0])

    def stop(self):
        self.queue.put(None)


def main(args=None):
    parser = argparse.ArgumentParser(prog='midifilter', description=__doc__)
    padd = parser.add_argument
    padd('-m', '--mpresstocc', action="store_true",
         help='Map mono pressure (channel aftertouch) to CC')
    padd('-r', '--mapccrange', action="store_true",
         help='Map controller value range to min/max value range')
    padd('-t', '--transpose', action="store_true",
         help='Transpose note on/off event note values')
    padd('-i', '--inport',
         help='MIDI input port number (default: ask)')
    padd('-o', '--outport',
         help='MIDI output port number (default: ask)')
    padd('-v', '--verbose', action="store_true",
         help='verbose output')
    padd('filterargs', nargs="*", type=int,
         help='MIDI filter argument(s)')

    args = parser.parse_args(args)

    logging.basicConfig(format="%(name)s: %(levelname)s - %(message)s",
                        level=logging.DEBUG if args.verbose else logging.INFO)

    try:
        midiin, inport_name = open_midiport(args.inport, "input")
        midiout, outport_name = open_midiport(args.outport, "output")
    except IOError as exc:
        print(exc)
        return 1
    except (EOFError, KeyboardInterrupt):
        return 0

    filters = []
    #filters = [CCToBankChange(cc=99, channel=15, msb=0, lsb=1, program=99)]

    if args.transpose:
        filters.append(Transpose(transpose=args.filterargs[0]))
    if args.mpresstocc:
        filters.append(MonoPressureToCC(cc=args.filterargs[0]))
    if args.mapccrange:
        filters.append(MapControllerValue(*args.filterargs))

    dispatcher = MidiDispatcher(midiin, midiout, *filters)

    print("Entering main loop. Press Control-C to exit.")
    try:
        dispatcher.start()
        while True:
            time.sleep(1)
    except KeyboardInterrupt:
        dispatcher.stop()
        dispatcher.join()
        print('')
    finally:
        print("Exit.")

        midiin.close_port()
        midiout.close_port()

        del midiin
        del midiout

    return 0


if __name__ == '__main__':
    sys.exit(main() or 0)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Unit tests for the rtmidi module."""

import gc
import unittest

import rtmidi


class TestDelete(unittest.TestCase):

    def setUp(self):
        self.midiout = rtmidi.MidiOut()
        self.midiin = rtmidi.MidiIn()

    def test_midiin_delete(self):
        midiin_ports = self.midiin.get_ports()
        print("Input ports:")
        print(midiin_ports)

        ports_init = self.midiout.get_ports()
        print("output ports initially:")
        print(ports_init)

        if midiin_ports:
            self.midiin.open_port(0)
        else:
            self.midiin.open_virtual_port("My virtual output")

        ports_before = self.midiout.get_ports()

        self.assertEqual(len(ports_before), len(ports_init) + 1)
        print("Output ports BEFORE deleting MidiIn instance:")
        print(ports_before)

        self.midiin.delete()

        ports_after = self.midiout.get_ports()
        print("Output ports AFTER deleting MidiIn instance:")
        print(ports_after)

        self.assertEqual(set(ports_init), set(ports_after))

    def test_midiout_delete(self):
        midiout_ports = self.midiout.get_ports()
        print("Output ports:")
        print(midiout_ports)

        ports_init = self.midiin.get_ports()
        print("Input ports initially:")
        print(ports_init)

        if midiout_ports:
            self.midiout.open_port(0)
        else:
            self.midiout.open_virtual_port("My virtual output")

        ports_before = self.midiin.get_ports()

        self.assertEqual(len(ports_before), len(ports_init) + 1)
        print("Input ports BEFORE deleting MidiOut instance:")
        print(ports_before)

        self.midiout.delete()

        ports_after = self.midiin.get_ports()
        print("Input ports AFTER deleting MidiOut instance:")
        print(ports_after)

        self.assertEqual(set(ports_init), set(ports_after))

    def test_double_delete(self):
        self.assertFalse(self.midiout.is_deleted)
        self.midiout.delete()
        self.assertTrue(self.midiout.is_deleted)
        self.midiout.delete()
        self.assertTrue(self.midiout.is_deleted)

    def test_del_after_delete(self):
        self.midiout.delete()
        self.assertTrue(self.midiout.is_deleted)
        self.assertTrue(hasattr(self, 'midiout'))
        del self.midiout
        gc.collect()
        self.assertFalse(hasattr(self, 'midiout'))


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Setup file for the Cython rtmidi wrapper."""

from __future__ import print_function

import subprocess
import sys

from ctypes.util import find_library
from os.path import dirname, exists, join

from setuptools import setup  # needs to stay before the imports below!
import distutils
from distutils.dist import DistributionMetadata
from distutils.extension import Extension
from distutils.version import StrictVersion

from fill_template import FillTemplate

try:
    from Cython.Build import cythonize
except ImportError:
    cythonize = None


JACK1_MIN_VERSION = StrictVersion('0.125.0')
JACK2_MIN_VERSION = StrictVersion('1.9.11')


def read(*args):
    return open(join(dirname(__file__), *args)).read()


def check_for_jack(define_macros, libraries):
    """Check for presence of jack library and set defines and libraries accordingly."""

    if find_library('jack'):
        define_macros.append(('__UNIX_JACK__', None))

        # Check version of jack whether it is "new" enough to have the
        # 'jack_port_rename' function:
        try:
            res = subprocess.check_output(['pkg-config', '--modversion', 'jack'])
            jv = StrictVersion(res.decode())
        except (subprocess.CalledProcessError, UnicodeError, ValueError):
            pass
        else:
            print("Detected JACK version %s." % jv, file=sys.stderr)
            if ((jv.version[0] == 0 and jv >= JACK1_MIN_VERSION) or
                    (jv.version[0] == 1 and jv >= JACK2_MIN_VERSION)):
                print("JACK version is recent enough to have 'jack_port_rename' function.",
                      file=sys.stderr)
                define_macros.append(('JACK_HAS_PORT_RENAME', None))

        libraries.append('jack')


class ToxTestCommand(distutils.cmd.Command):
    """Distutils command to run tests via tox with 'python setup.py test'.

    Please note that in this configuration tox uses the dependencies in
    `requirements/dev.txt`, the list of dependencies in `tests_require` in
    `setup.py` is ignored!

    See https://docs.python.org/3/distutils/apiref.html#creating-a-new-distutils-command
    for more documentation on custom distutils commands.

    """
    description = "Run tests via 'tox'."
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        self.announce("Running tests with 'tox'...", level=distutils.log.INFO)
        return subprocess.call(['tox'])


# source package structure
SRC_DIR = "src"
PKG_DIR = "rtmidi"

# Add custom distribution meta-data, avoids warning when running setup
DistributionMetadata.repository = None

# Read version number from version.py (without importing the 'rtmidi' package,
# since that would lead to hen-egg situation).
setup_opts = {}
exec(read(PKG_DIR, 'version.py'), {}, setup_opts)

# Add our own custom distutils command to create *.rst files from templates
# Template files are listed in setup.cfg
setup_opts.setdefault('cmdclass', {})
setup_opts['cmdclass']['filltmpl'] = FillTemplate
# Add custom test command
setup_opts['cmdclass']['test'] = ToxTestCommand

# Set up options for compiling the _rtmidi Extension
if cythonize:
    sources = [join(SRC_DIR, "_rtmidi.pyx"), join(SRC_DIR, "rtmidi", "RtMidi.cpp")]
elif exists(join(SRC_DIR, "_rtmidi.cpp")):
    cythonize = lambda x: x  # noqa
    sources = [join(SRC_DIR, "_rtmidi.cpp"), join(SRC_DIR, "rtmidi", "RtMidi.cpp")]
else:
    sys.exit("""\
Could not import Cython. Cython >= 0.28 is required to compile the Cython
source into the C++ source.

Install Cython from https://pypi.python.org/pypi/Cython or use the
pre-generated '_rtmidi.cpp' file from the python-rtmidi source distribution.
""")

define_macros = []
include_dirs = [join(SRC_DIR, "rtmidi")]
libraries = []
extra_link_args = []
extra_compile_args = []
alsa = coremidi = jack = winmm = True

if '--no-alsa' in sys.argv:
    alsa = False
    sys.argv.remove('--no-alsa')

if '--no-coremidi' in sys.argv:
    coremidi = False
    sys.argv.remove('--no-coremidi')

if '--no-jack' in sys.argv:
    jack = False
    sys.argv.remove('--no-jack')

if '--no-winmm' in sys.argv:
    winmm = False
    sys.argv.remove('--no-winmm')

if '--no-suppress-warnings' not in sys.argv:
    define_macros.append(('__RTMIDI_SILENCE_WARNINGS__', None))
else:
    sys.argv.remove('--no-suppress-warnings')


if sys.platform.startswith('linux'):
    if alsa and find_library('asound'):
        define_macros.append(("__LINUX_ALSA__", None))
        libraries.append('asound')

    if jack:
        check_for_jack(define_macros, libraries)

    if not find_library('pthread'):
        sys.exit("The 'pthread' library is required to build python-rtmidi on"
                 "Linux. Please install the libc6 development package.")

    libraries.append("pthread")
elif sys.platform.startswith('darwin'):
    if jack:
        check_for_jack(define_macros, libraries)

    if coremidi:
        define_macros.append(('__MACOSX_CORE__', None))
        extra_compile_args.append('-frtti')
        extra_link_args.extend([
            '-framework', 'CoreAudio',
            '-framework', 'CoreMIDI',
            '-framework', 'CoreFoundation'])
elif sys.platform.startswith('win'):
    extra_compile_args.append('/EHsc')

    if winmm:
        define_macros.append(('__WINDOWS_MM__', None))
        libraries.append("winmm")

else:
    print("WARNING: This operating system (%s) is not supported by RtMidi.\n"
          "Linux, macOS (OS X) (>= 10.5), Windows (XP, Vista, 7/8/10) are supported.\n"
          "Continuing and hoping for the best..." % sys.platform, file=sys.stderr)

# define _rtmidi Extension
extensions = [
    Extension(
        PKG_DIR + "._rtmidi",
        sources=sources,
        language="c++",
        define_macros=define_macros,
        include_dirs=include_dirs,
        libraries=libraries,
        extra_compile_args=extra_compile_args,
        extra_link_args=extra_link_args
    )
]

# Finally, set up our distribution
setup(
    packages=['rtmidi'],
    ext_modules=cythonize(extensions),
    tests_require=[],  # Test dependencies are handled by tox
    # On systems without a RTC (e.g. Raspberry Pi), system time will be the
    # Unix epoch when booted without network connection, which makes zip fail,
    # because it does not support dates < 1980-01-01.
    **setup_opts
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Tests for the error callback"""

import unittest
try:
    from unittest import mock
except ImportError:
    import mock

import rtmidi


class TestErrorCallback(unittest.TestCase):

    INVALID_PORT_NUMBER = 9999
    MIDI_OUT_ERROR_USER_DATA = 'midiOutError'
    MIDI_IN_ERROR_USER_DATA = 'midiInError'

    def setUp(self):
        self.midi_out = rtmidi.MidiOut()
        self.midi_in = rtmidi.MidiIn()

    def test_default_error_handler(self):
        with self.assertRaises(rtmidi.InvalidPortError):
            self.midi_out.open_port(self.INVALID_PORT_NUMBER)

    def test_midiout_error_callback(self):
        errcb = mock.Mock()
        self.midi_out.set_error_callback(errcb, self.MIDI_OUT_ERROR_USER_DATA)
        self.midi_out.open_port(self.INVALID_PORT_NUMBER)
        errcb.assert_called_with(rtmidi.ERRORTYPE_INVALID_PARAMETER,
                                 mock.ANY,
                                 self.MIDI_OUT_ERROR_USER_DATA)

    def test_midiin_error_callback(self):
        errcb = mock.Mock()
        self.midi_in.set_error_callback(errcb, self.MIDI_IN_ERROR_USER_DATA)
        self.midi_in.open_port(self.INVALID_PORT_NUMBER)
        errcb.assert_called_with(rtmidi.ERRORTYPE_INVALID_PARAMETER,
                                 mock.ANY,
                                 self.MIDI_IN_ERROR_USER_DATA)

    def test_error_callback_without_user_data(self):
        errcb = mock.Mock()
        self.midi_out.set_error_callback(errcb)
        self.midi_out.open_port(self.INVALID_PORT_NUMBER)
        errcb.assert_called_with(rtmidi.ERRORTYPE_INVALID_PARAMETER,
                                 mock.ANY,
                                 None)

    def test_cancel_error_callback(self):
        errcb = mock.Mock()
        self.midi_out.set_error_callback(errcb)
        self.midi_out.cancel_error_callback()
        try:
            self.midi_out.open_port(self.INVALID_PORT_NUMBER)
        except rtmidi.InvalidPortError:
            pass
        errcb.assert_not_called()


if __name__ == '__main__':
    unittest.main()
